@@ -11,6 +11,7 @@ mod compute;
 
 pub use array::*;
 pub use compress::*;
+use vortex_buffer::{Buffer, BufferMut};
 
 const SAMPLE_SIZE: usize = 32;
 
@@ -34,7 +35,7 @@ mod private {
 }
 
 pub trait ALPFloat: private::Sealed + Float + Display + 'static {
-    type ALPInt: PrimInt + Display + ToPrimitive;
+    type ALPInt: PrimInt + Display + ToPrimitive + Copy;
 
     const FRACTIONAL_BITS: u8;
     const MAX_EXPONENT: u8;
@@ -114,12 +115,12 @@ pub trait ALPFloat: private::Sealed + Float + Display + 'static {
     fn encode(
         values: &[Self],
         exponents: Option<Exponents>,
-    ) -> (Exponents, Vec<Self::ALPInt>, Vec<u64>, Vec<Self>) {
+    ) -> (Exponents, Buffer<Self::ALPInt>, Buffer<u64>, Buffer<Self>) {
         let exp = exponents.unwrap_or_else(|| Self::find_best_exponents(values));
 
-        let mut encoded_output = Vec::with_capacity(values.len());
-        let mut patch_indices = Vec::new();
-        let mut patch_values = Vec::new();
+        let mut encoded_output = BufferMut::<Self::ALPInt>::with_capacity(values.len());
+        let mut patch_indices = BufferMut::<u64>::with_capacity(values.len());
+        let mut patch_values = BufferMut::<Self>::with_capacity(values.len());
         let mut fill_value: Option<Self::ALPInt> = None;
 
         // this is intentionally branchless
@@ -136,17 +137,34 @@ pub trait ALPFloat: private::Sealed + Float + Display + 'static {
             );
         }
 
-        (exp, encoded_output, patch_indices, patch_values)
+        (
+            exp,
+            encoded_output.freeze(),
+            patch_indices.freeze(),
+            patch_values.freeze(),
+        )
     }
 
     #[inline]
-    fn encode_single(value: Self, exponents: Exponents) -> Result<Self::ALPInt, Self> {
+    fn encode_single(value: Self, exponents: Exponents) -> Option<Self::ALPInt> {
         let encoded = unsafe { Self::encode_single_unchecked(value, exponents) };
         let decoded = Self::decode_single(encoded, exponents);
         if decoded == value {
-            return Ok(encoded);
+            return Some(encoded);
         }
-        Err(value)
+        None
+    }
+
+    fn encode_above(value: Self, exponents: Exponents) -> Self::ALPInt {
+        (value * Self::F10[exponents.e as usize] * Self::IF10[exponents.f as usize])
+            .ceil()
+            .as_int()
+    }
+
+    fn encode_below(value: Self, exponents: Exponents) -> Self::ALPInt {
+        (value * Self::F10[exponents.e as usize] * Self::IF10[exponents.f as usize])
+            .floor()
+            .as_int()
     }
 
     fn decode(encoded: &[Self::ALPInt], exponents: Exponents) -> Vec<Self> {
@@ -157,7 +175,11 @@ pub trait ALPFloat: private::Sealed + Float + Display + 'static {
         values
     }
 
-    #[inline]
+    fn decode_buffer(encoded: BufferMut<Self::ALPInt>, exponents: Exponents) -> BufferMut<Self> {
+        encoded.map_each(move |encoded| Self::decode_single(encoded, exponents))
+    }
+
+    #[inline(always)]
     fn decode_single(encoded: Self::ALPInt, exponents: Exponents) -> Self {
         Self::from_int(encoded) * Self::F10[exponents.f as usize] * Self::IF10[exponents.e as usize]
     }
@@ -173,12 +195,13 @@ pub trait ALPFloat: private::Sealed + Float + Display + 'static {
     }
 }
 
+#[allow(clippy::cast_possible_truncation)]
 fn encode_chunk_unchecked<T: ALPFloat>(
     chunk: &[T],
     exp: Exponents,
-    encoded_output: &mut Vec<T::ALPInt>,
-    patch_indices: &mut Vec<u64>,
-    patch_values: &mut Vec<T>,
+    encoded_output: &mut BufferMut<T::ALPInt>,
+    patch_indices: &mut BufferMut<u64>,
+    patch_values: &mut BufferMut<T>,
     fill_value: &mut Option<T::ALPInt>,
 ) {
     let num_prev_encoded = encoded_output.len();
@@ -280,6 +303,7 @@ impl ALPFloat for f32 {
     ];
 
     #[inline(always)]
+    #[allow(clippy::cast_possible_truncation)]
     fn as_int(self) -> Self::ALPInt {
         self as _
     }
@@ -351,6 +375,7 @@ impl ALPFloat for f64 {
     ];
 
     #[inline(always)]
+    #[allow(clippy::cast_possible_truncation)]
     fn as_int(self) -> Self::ALPInt {
         self as _
     }
