@@ -1,20 +1,29 @@
-use std::fmt::{Debug, Display};
+use std::fmt::Debug;
 
 use serde::{Deserialize, Serialize};
-use vortex_array::array::{PrimitiveArray, SparseArray};
-use vortex_array::encoding::ids;
-use vortex_array::stats::{StatisticsVTable, StatsSet};
-use vortex_array::validity::{ArrayValidity, LogicalValidity, ValidityVTable};
-use vortex_array::visitor::{ArrayVisitor, VisitorVTable};
+use vortex_array::array::PrimitiveArray;
+use vortex_array::patches::{Patches, PatchesMetadata};
+use vortex_array::stats::StatsSet;
+use vortex_array::validity::Validity;
+use vortex_array::visitor::ArrayVisitor;
+use vortex_array::vtable::{
+    CanonicalVTable, StatisticsVTable, ValidateVTable, ValidityVTable, VisitorVTable,
+};
 use vortex_array::{
-    impl_encoding, ArrayDType, ArrayData, ArrayLen, ArrayTrait, Canonical, IntoCanonical,
+    encoding_ids, impl_encoding, Array, Canonical, IntoArrayVariant, SerdeMetadata,
 };
 use vortex_dtype::{DType, Nullability, PType};
 use vortex_error::{vortex_bail, VortexExpect, VortexResult};
+use vortex_mask::Mask;
 
 use crate::alp_rd::alp_rd_decode;
 
-impl_encoding!("vortex.alprd", ids::ALP_RD, ALPRD);
+impl_encoding!(
+    "vortex.alprd",
+    encoding_ids::ALP_RD,
+    ALPRD,
+    SerdeMetadata<ALPRDMetadata>
+);
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct ALPRDMetadata {
@@ -22,23 +31,17 @@ pub struct ALPRDMetadata {
     dict_len: u8,
     dict: [u16; 8],
     left_parts_ptype: PType,
-    has_exceptions: bool,
-}
-
-impl Display for ALPRDMetadata {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        Debug::fmt(self, f)
-    }
+    patches: Option<PatchesMetadata>,
 }
 
 impl ALPRDArray {
     pub fn try_new(
         dtype: DType,
-        left_parts: ArrayData,
+        left_parts: Array,
         left_parts_dict: impl AsRef<[u16]>,
-        right_parts: ArrayData,
+        right_parts: Array,
         right_bit_width: u8,
-        left_parts_exceptions: Option<ArrayData>,
+        left_parts_patches: Option<Patches>,
     ) -> VortexResult<Self> {
         if !dtype.is_float() {
             vortex_bail!("ALPRDArray given invalid DType ({dtype})");
@@ -75,16 +78,21 @@ impl ALPRDArray {
             vortex_bail!(MismatchedTypes: "non-nullable uint", right_parts.dtype());
         }
 
-        let mut children = vec![left_parts, right_parts];
-        let has_exceptions = left_parts_exceptions.is_some();
+        let mut children = vec![left_parts.clone(), right_parts];
 
-        if let Some(exceptions) = left_parts_exceptions {
-            // Enforce that the exceptions are SparseArray so that we have access to indices and values.
-            if exceptions.encoding().id().code() != ids::SPARSE {
-                vortex_bail!("left_parts_exceptions must be SparseArray encoded");
-            }
-            children.push(exceptions);
-        }
+        let patches = left_parts_patches
+            .map(|patches| {
+                if patches.values().dtype().is_nullable() {
+                    vortex_bail!("patches must be non-nullable: {}", patches.values());
+                }
+                let metadata =
+                    patches.to_metadata(left_parts.len(), &left_parts.dtype().as_nonnullable());
+                let (_, indices, values) = patches.into_parts();
+                children.push(indices);
+                children.push(values);
+                metadata
+            })
+            .transpose()?;
 
         let mut dict = [0u16; 8];
         for (idx, v) in left_parts_dict.as_ref().iter().enumerate() {
@@ -94,14 +102,15 @@ impl ALPRDArray {
         Self::try_from_parts(
             dtype,
             len,
-            ALPRDMetadata {
+            SerdeMetadata(ALPRDMetadata {
                 right_bit_width,
                 dict_len: left_parts_dict.as_ref().len() as u8,
                 dict,
                 left_parts_ptype,
-                has_exceptions,
-            },
-            children.into(),
+                patches,
+            }),
+            None,
+            Some(children.into()),
             StatsSet::default(),
         )
     }
@@ -134,42 +143,50 @@ impl ALPRDArray {
         )
     }
 
-    /// The dtype of the exceptions of the left parts of the array.
+    /// The dtype of the patches of the left parts of the array.
     #[inline]
-    fn left_parts_exceptions_dtype(&self) -> DType {
-        DType::Primitive(self.metadata().left_parts_ptype, Nullability::Nullable)
+    fn left_parts_patches_dtype(&self) -> DType {
+        DType::Primitive(self.metadata().left_parts_ptype, Nullability::NonNullable)
     }
 
     /// The leftmost (most significant) bits of the floating point values stored in the array.
     ///
     /// These are bit-packed and dictionary encoded, and cannot directly be interpreted without
     /// the metadata of this array.
-    pub fn left_parts(&self) -> ArrayData {
+    pub fn left_parts(&self) -> Array {
         self.as_ref()
             .child(0, &self.left_parts_dtype(), self.len())
             .vortex_expect("ALPRDArray: left_parts child")
     }
 
     /// The rightmost (least significant) bits of the floating point values stored in the array.
-    pub fn right_parts(&self) -> ArrayData {
+    pub fn right_parts(&self) -> Array {
         self.as_ref()
             .child(1, &self.right_parts_dtype(), self.len())
             .vortex_expect("ALPRDArray: right_parts child")
     }
 
     /// Patches of left-most bits.
-    pub fn left_parts_exceptions(&self) -> Option<ArrayData> {
-        self.metadata().has_exceptions.then(|| {
-            self.as_ref()
-                .child(2, &self.left_parts_exceptions_dtype(), self.len())
-                .vortex_expect("ALPRDArray: left_parts_exceptions child")
+    pub fn left_parts_patches(&self) -> Option<Patches> {
+        self.metadata().patches.as_ref().map(|metadata| {
+            Patches::new(
+                self.len(),
+                self.as_ref()
+                    .child(2, &metadata.indices_dtype(), metadata.len())
+                    .vortex_expect("ALPRDArray: patch indices"),
+                self.as_ref()
+                    .child(3, &self.left_parts_patches_dtype(), metadata.len())
+                    .vortex_expect("ALPRDArray: patch values"),
+            )
         })
     }
 
     /// The dictionary that maps the codes in `left_parts` into bit patterns.
     #[inline]
-    pub fn left_parts_dict(&self) -> &[u16] {
-        &self.metadata().dict[0..self.metadata().dict_len as usize]
+    pub fn left_parts_dict(&self) -> Vec<u16> {
+        // FIXME(ngates): either have metadata that can be a view over the bytes.
+        //  Or move dictionary into a buffer.
+        self.metadata().dict[0..self.metadata().dict_len as usize].to_vec()
     }
 
     #[inline]
@@ -178,57 +195,35 @@ impl ALPRDArray {
     }
 }
 
-impl IntoCanonical for ALPRDArray {
-    fn into_canonical(self) -> VortexResult<Canonical> {
-        let left_parts = self.left_parts().into_canonical()?.into_primitive()?;
-        let right_parts = self.right_parts().into_canonical()?.into_primitive()?;
+impl CanonicalVTable<ALPRDArray> for ALPRDEncoding {
+    fn into_canonical(&self, array: ALPRDArray) -> VortexResult<Canonical> {
+        let left_parts = array.left_parts().into_primitive()?;
+        let right_parts = array.right_parts().into_primitive()?;
 
         // Decode the left_parts using our builtin dictionary.
-        let left_parts_dict = &self.metadata().dict[0..self.metadata().dict_len as usize];
+        let left_parts_dict = &array.metadata().dict[0..array.metadata().dict_len as usize];
 
-        let exc_pos: Vec<u64>;
-        let exc_u16: PrimitiveArray;
-
-        if let Some(left_parts_exceptions) = self.left_parts_exceptions() {
-            let left_parts_exceptions = SparseArray::try_from(left_parts_exceptions)
-                .vortex_expect("ALPRDArray: exceptions must be SparseArray encoded");
-            exc_pos = left_parts_exceptions
-                .resolved_indices()
-                .into_iter()
-                .map(|v| v as _)
-                .collect();
-            exc_u16 = left_parts_exceptions
-                .values()
-                .into_canonical()?
-                .into_primitive()?;
-        } else {
-            exc_pos = Vec::new();
-            exc_u16 = PrimitiveArray::from(Vec::<u16>::new());
-        }
-
-        let decoded_array = if self.is_f32() {
-            PrimitiveArray::from_vec(
+        let decoded_array = if array.is_f32() {
+            PrimitiveArray::new(
                 alp_rd_decode::<f32>(
-                    left_parts.maybe_null_slice::<u16>(),
+                    left_parts.into_buffer::<u16>(),
                     left_parts_dict,
-                    self.metadata().right_bit_width,
-                    right_parts.maybe_null_slice::<u32>(),
-                    &exc_pos,
-                    exc_u16.maybe_null_slice::<u16>(),
-                ),
-                self.logical_validity().into_validity(),
+                    array.metadata().right_bit_width,
+                    right_parts.into_buffer_mut::<u32>(),
+                    array.left_parts_patches(),
+                )?,
+                Validity::from_mask(array.logical_validity()?, array.dtype().nullability()),
             )
         } else {
-            PrimitiveArray::from_vec(
+            PrimitiveArray::new(
                 alp_rd_decode::<f64>(
-                    left_parts.maybe_null_slice::<u16>(),
+                    left_parts.into_buffer::<u16>(),
                     left_parts_dict,
-                    self.metadata().right_bit_width,
-                    right_parts.maybe_null_slice::<u64>(),
-                    &exc_pos,
-                    exc_u16.maybe_null_slice::<u16>(),
-                ),
-                self.logical_validity().into_validity(),
+                    array.metadata().right_bit_width,
+                    right_parts.into_buffer_mut::<u64>(),
+                    array.left_parts_patches(),
+                )?,
+                Validity::from_mask(array.logical_validity()?, array.dtype().nullability()),
             )
         };
 
@@ -237,12 +232,12 @@ impl IntoCanonical for ALPRDArray {
 }
 
 impl ValidityVTable<ALPRDArray> for ALPRDEncoding {
-    fn is_valid(&self, array: &ALPRDArray, index: usize) -> bool {
+    fn is_valid(&self, array: &ALPRDArray, index: usize) -> VortexResult<bool> {
         // Use validity from left_parts
         array.left_parts().is_valid(index)
     }
 
-    fn logical_validity(&self, array: &ALPRDArray) -> LogicalValidity {
+    fn logical_validity(&self, array: &ALPRDArray) -> VortexResult<Mask> {
         // Use validity from left_parts
         array.left_parts().logical_validity()
     }
@@ -252,8 +247,8 @@ impl VisitorVTable<ALPRDArray> for ALPRDEncoding {
     fn accept(&self, array: &ALPRDArray, visitor: &mut dyn ArrayVisitor) -> VortexResult<()> {
         visitor.visit_child("left_parts", &array.left_parts())?;
         visitor.visit_child("right_parts", &array.right_parts())?;
-        if let Some(left_parts_exceptions) = array.left_parts_exceptions() {
-            visitor.visit_child("left_parts_exceptions", &left_parts_exceptions)
+        if let Some(patches) = array.left_parts_patches() {
+            visitor.visit_patches(&patches)
         } else {
             Ok(())
         }
@@ -262,20 +257,38 @@ impl VisitorVTable<ALPRDArray> for ALPRDEncoding {
 
 impl StatisticsVTable<ALPRDArray> for ALPRDEncoding {}
 
-impl ArrayTrait for ALPRDArray {}
+impl ValidateVTable<ALPRDArray> for ALPRDEncoding {}
 
 #[cfg(test)]
 mod test {
     use rstest::rstest;
     use vortex_array::array::PrimitiveArray;
-    use vortex_array::{IntoArrayData, IntoCanonical};
+    use vortex_array::patches::PatchesMetadata;
+    use vortex_array::test_harness::check_metadata;
+    use vortex_array::{IntoArrayVariant, SerdeMetadata};
+    use vortex_dtype::PType;
 
-    use crate::{alp_rd, ALPRDFloat};
+    use crate::{alp_rd, ALPRDFloat, ALPRDMetadata};
+
+    #[cfg_attr(miri, ignore)]
+    #[test]
+    fn test_alprd_metadata() {
+        check_metadata(
+            "alprd.metadata",
+            SerdeMetadata(ALPRDMetadata {
+                right_bit_width: u8::MAX,
+                patches: Some(PatchesMetadata::new(usize::MAX, PType::U64)),
+                dict: [0u16; 8],
+                left_parts_ptype: PType::U64,
+                dict_len: 8,
+            }),
+        );
+    }
 
     #[rstest]
     #[case(vec![0.1f32.next_up(); 1024], 1.123_848_f32)]
     #[case(vec![0.1f64.next_up(); 1024], 1.123_848_591_110_992_f64)]
-    fn test_array_encode_with_nulls_and_exceptions<T: ALPRDFloat>(
+    fn test_array_encode_with_nulls_and_patches<T: ALPRDFloat>(
         #[case] reals: Vec<T>,
         #[case] seed: T,
     ) {
@@ -287,21 +300,16 @@ mod test {
         reals[900] = None;
 
         // Create a new array from this.
-        let real_array = PrimitiveArray::from_nullable_vec(reals.clone());
+        let real_array = PrimitiveArray::from_option_iter(reals.iter().cloned());
 
-        // Pick a seed that we know will trigger lots of exceptions.
+        // Pick a seed that we know will trigger lots of patches.
         let encoder: alp_rd::RDEncoder = alp_rd::RDEncoder::new(&[seed.powi(-2)]);
 
         let rd_array = encoder.encode(&real_array);
 
-        let decoded = rd_array
-            .into_array()
-            .into_canonical()
-            .unwrap()
-            .into_primitive()
-            .unwrap();
+        let decoded = rd_array.into_primitive().unwrap();
 
         let maybe_null_reals: Vec<T> = reals.into_iter().map(|v| v.unwrap_or_default()).collect();
-        assert_eq!(decoded.maybe_null_slice::<T>(), &maybe_null_reals);
+        assert_eq!(decoded.as_slice::<T>(), &maybe_null_reals);
     }
 }
