@@ -1,5 +1,6 @@
-use vortex_array::stats::{ArrayStatistics, Stat, StatisticsVTable, StatsSet};
-use vortex_array::{ArrayLen, IntoArrayVariant};
+use vortex_array::stats::{Stat, StatsSet};
+use vortex_array::vtable::StatisticsVTable;
+use vortex_array::IntoArrayVariant;
 use vortex_error::VortexResult;
 
 use super::{ByteBoolArray, ByteBoolEncoding};
@@ -12,21 +13,16 @@ impl StatisticsVTable<ByteBoolArray> for ByteBoolEncoding {
 
         // TODO(adamgs): This is slightly wasteful and could be optimized in the future
         let bools = array.as_ref().clone().into_bool()?;
-        Ok(StatsSet::from_iter(
-            bools
-                .statistics()
-                .compute(stat)
-                .into_iter()
-                .map(|value| (stat, value)),
-        ))
+        Ok(bools
+            .statistics()
+            .compute(stat)
+            .map(|value| StatsSet::of(stat, value))
+            .unwrap_or_default())
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use vortex_array::stats::ArrayStatistics;
-    use vortex_dtype::{DType, Nullability};
-    use vortex_scalar::Scalar;
 
     use super::*;
 
@@ -92,14 +88,8 @@ mod tests {
         assert!(!bool_arr.statistics().compute_is_strict_sorted().unwrap());
         assert!(bool_arr.statistics().compute_is_sorted().unwrap());
         assert!(bool_arr.statistics().compute_is_constant().unwrap());
-        assert_eq!(
-            bool_arr.statistics().compute(Stat::Min).unwrap(),
-            Scalar::null(DType::Bool(Nullability::Nullable))
-        );
-        assert_eq!(
-            bool_arr.statistics().compute(Stat::Max).unwrap(),
-            Scalar::null(DType::Bool(Nullability::Nullable))
-        );
+        assert!(bool_arr.statistics().compute(Stat::Min).is_none());
+        assert!(bool_arr.statistics().compute(Stat::Max).is_none());
         assert_eq!(bool_arr.statistics().compute_run_count().unwrap(), 1);
         assert_eq!(bool_arr.statistics().compute_true_count().unwrap(), 0);
     }
