@@ -1,4 +1,5 @@
-use vortex_array::stats::{ArrayStatistics, Stat, StatisticsVTable, StatsSet};
+use vortex_array::stats::{Stat, StatsSet};
+use vortex_array::vtable::StatisticsVTable;
 use vortex_error::VortexResult;
 
 use crate::{DictArray, DictEncoding};
