@@ -1,38 +1,39 @@
-use std::fmt::{Debug, Display};
+use std::fmt::Debug;
 
 pub use compress::*;
-use serde::{Deserialize, Serialize};
 use vortex_array::array::PrimitiveArray;
-use vortex_array::encoding::ids;
-use vortex_array::stats::{StatisticsVTable, StatsSet};
-use vortex_array::validity::{LogicalValidity, Validity, ValidityMetadata, ValidityVTable};
-use vortex_array::variants::{ArrayVariants, PrimitiveArrayTrait};
-use vortex_array::visitor::{ArrayVisitor, VisitorVTable};
-use vortex_array::{
-    impl_encoding, ArrayDType, ArrayData, ArrayLen, ArrayTrait, Canonical, IntoArrayData,
-    IntoCanonical,
+use vortex_array::stats::StatsSet;
+use vortex_array::validity::{Validity, ValidityMetadata};
+use vortex_array::variants::PrimitiveArrayTrait;
+use vortex_array::visitor::ArrayVisitor;
+use vortex_array::vtable::{
+    CanonicalVTable, StatisticsVTable, ValidateVTable, ValidityVTable, VariantsVTable,
+    VisitorVTable,
 };
+use vortex_array::{encoding_ids, impl_encoding, Array, Canonical, IntoArray, RkyvMetadata};
+use vortex_buffer::Buffer;
 use vortex_dtype::{match_each_unsigned_integer_ptype, NativePType};
 use vortex_error::{vortex_bail, vortex_panic, VortexExpect as _, VortexResult};
+use vortex_mask::Mask;
 
 mod compress;
 mod compute;
 
-impl_encoding!("fastlanes.delta", ids::FL_DELTA, Delta);
+impl_encoding!(
+    "fastlanes.delta",
+    encoding_ids::FL_DELTA,
+    Delta,
+    RkyvMetadata<DeltaMetadata>
+);
 
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, rkyv::Archive, rkyv::Serialize, rkyv::Deserialize)]
+#[repr(C)]
 pub struct DeltaMetadata {
     validity: ValidityMetadata,
     deltas_len: u64,
     offset: u16, // must be <1024
 }
 
-impl Display for DeltaMetadata {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        Debug::fmt(self, f)
-    }
-}
-
 /// A FastLanes-style delta-encoded array of primitive values.
 ///
 /// A [`DeltaArray`] comprises a sequence of _chunks_ each representing 1,024 delta-encoded values,
@@ -66,8 +67,12 @@ impl Display for DeltaMetadata {
 /// If the chunk physically has fewer than 1,024 values, then it is stored as a traditional,
 /// non-SIMD-amenable, delta-encoded vector.
 impl DeltaArray {
+    // TODO(ngates): remove constructing from vec
     pub fn try_from_vec<T: NativePType>(vec: Vec<T>) -> VortexResult<Self> {
-        Self::try_from_primitive_array(&PrimitiveArray::from(vec))
+        Self::try_from_primitive_array(&PrimitiveArray::new(
+            Buffer::copy_from(vec),
+            Validity::NonNullable,
+        ))
     }
 
     pub fn try_from_primitive_array(array: &PrimitiveArray) -> VortexResult<Self> {
@@ -81,8 +86,8 @@ impl DeltaArray {
     }
 
     pub fn try_from_delta_compress_parts(
-        bases: ArrayData,
-        deltas: ArrayData,
+        bases: Array,
+        deltas: Array,
         validity: Validity,
     ) -> VortexResult<Self> {
         let logical_len = deltas.len();
@@ -90,8 +95,8 @@ impl DeltaArray {
     }
 
     pub fn try_new(
-        bases: ArrayData,
-        deltas: ArrayData,
+        bases: Array,
+        deltas: Array,
         validity: Validity,
         offset: usize,
         logical_len: usize,
@@ -136,8 +141,9 @@ impl DeltaArray {
         let delta = Self::try_from_parts(
             dtype,
             logical_len,
-            metadata,
-            children.into(),
+            RkyvMetadata(metadata),
+            None,
+            Some(children.into()),
             StatsSet::default(),
         )?;
 
@@ -164,14 +170,14 @@ impl DeltaArray {
     }
 
     #[inline]
-    pub fn bases(&self) -> ArrayData {
+    pub fn bases(&self) -> Array {
         self.as_ref()
             .child(0, self.dtype(), self.bases_len())
             .vortex_expect("DeltaArray is missing bases child array")
     }
 
     #[inline]
-    pub fn deltas(&self) -> ArrayData {
+    pub fn deltas(&self) -> Array {
         self.as_ref()
             .child(1, self.dtype(), self.deltas_len())
             .vortex_expect("DeltaArray is missing deltas child array")
@@ -216,28 +222,28 @@ impl DeltaArray {
     }
 }
 
-impl ArrayTrait for DeltaArray {}
+impl ValidateVTable<DeltaArray> for DeltaEncoding {}
 
-impl ArrayVariants for DeltaArray {
-    fn as_primitive_array(&self) -> Option<&dyn PrimitiveArrayTrait> {
-        Some(self)
+impl VariantsVTable<DeltaArray> for DeltaEncoding {
+    fn as_primitive_array<'a>(&self, array: &'a DeltaArray) -> Option<&'a dyn PrimitiveArrayTrait> {
+        Some(array)
     }
 }
 
 impl PrimitiveArrayTrait for DeltaArray {}
 
-impl IntoCanonical for DeltaArray {
-    fn into_canonical(self) -> VortexResult<Canonical> {
-        delta_decompress(self).map(Canonical::Primitive)
+impl CanonicalVTable<DeltaArray> for DeltaEncoding {
+    fn into_canonical(&self, array: DeltaArray) -> VortexResult<Canonical> {
+        delta_decompress(array).map(Canonical::Primitive)
     }
 }
 
 impl ValidityVTable<DeltaArray> for DeltaEncoding {
-    fn is_valid(&self, array: &DeltaArray, index: usize) -> bool {
+    fn is_valid(&self, array: &DeltaArray, index: usize) -> VortexResult<bool> {
         array.validity().is_valid(index)
     }
 
-    fn logical_validity(&self, array: &DeltaArray) -> LogicalValidity {
+    fn logical_validity(&self, array: &DeltaArray) -> VortexResult<Mask> {
         array.validity().to_logical(array.len())
     }
 }
@@ -250,3 +256,25 @@ impl VisitorVTable<DeltaArray> for DeltaEncoding {
 }
 
 impl StatisticsVTable<DeltaArray> for DeltaEncoding {}
+
+#[cfg(test)]
+mod test {
+    use vortex_array::test_harness::check_metadata;
+    use vortex_array::validity::ValidityMetadata;
+    use vortex_array::RkyvMetadata;
+
+    use crate::DeltaMetadata;
+
+    #[cfg_attr(miri, ignore)]
+    #[test]
+    fn test_delta_metadata() {
+        check_metadata(
+            "delta.metadata",
+            RkyvMetadata(DeltaMetadata {
+                offset: u16::MAX,
+                validity: ValidityMetadata::AllValid,
+                deltas_len: u64::MAX,
+            }),
+        );
+    }
+}
