@@ -4,9 +4,10 @@ use pyo3::exceptions::PyValueError;
 use pyo3::prelude::*;
 use pyo3::types::{IntoPyDict, PyInt, PyList};
 use vortex::array::ChunkedArray;
-use vortex::compute::unary::{fill_forward, scalar_at};
-use vortex::compute::{compare, slice, take, FilterMask, Operator, TakeOptions};
-use vortex::{ArrayDType, ArrayData, IntoCanonical};
+use vortex::arrow::{infer_data_type, IntoArrowArray};
+use vortex::compute::{compare, fill_forward, scalar_at, slice, take, Operator};
+use vortex::mask::Mask;
+use vortex::Array;
 
 use crate::dtype::PyDType;
 use crate::python_repr::PythonRepr;
@@ -77,15 +78,15 @@ use crate::scalar::scalar_into_py;
 ///   true
 /// ]
 pub struct PyArray {
-    inner: ArrayData,
+    inner: Array,
 }
 
 impl PyArray {
-    pub fn new(inner: ArrayData) -> PyArray {
+    pub fn new(inner: Array) -> PyArray {
         PyArray { inner }
     }
 
-    pub fn unwrap(&self) -> &ArrayData {
+    pub fn unwrap(&self) -> &Array {
         &self.inner
     }
 }
@@ -119,12 +120,13 @@ impl PyArray {
         let vortex = &self_.inner;
 
         if let Ok(chunked_array) = ChunkedArray::try_from(vortex.clone()) {
+            // We figure out a single Arrow Data Type to convert all chunks into, otherwise
+            // the preferred type of each chunk may be different.
+            let arrow_dtype = infer_data_type(chunked_array.dtype())?;
+
             let chunks: Vec<ArrayRef> = chunked_array
                 .chunks()
-                .map(|chunk| -> PyResult<ArrayRef> {
-                    let canonical = chunk.into_canonical()?;
-                    Ok(canonical.into_arrow()?)
-                })
+                .map(|chunk| -> PyResult<ArrayRef> { Ok(chunk.into_arrow(&arrow_dtype)?) })
                 .collect::<PyResult<Vec<ArrayRef>>>()?;
             if chunks.is_empty() {
                 return Err(PyValueError::new_err("No chunks in array"));
@@ -144,8 +146,7 @@ impl PyArray {
         } else {
             Ok(vortex
                 .clone()
-                .into_canonical()
-                .and_then(|arr| arr.into_arrow())?
+                .into_arrow_preferred()?
                 .into_data()
                 .to_pyarrow(py)?
                 .into_bound(py))
@@ -162,7 +163,7 @@ impl PyArray {
 
     #[getter]
     fn encoding(&self) -> String {
-        self.inner.encoding().id().to_string()
+        self.inner.encoding().to_string()
     }
 
     #[getter]
@@ -267,8 +268,7 @@ impl PyArray {
     /// ]
     fn filter(&self, filter: &Bound<PyArray>) -> PyResult<PyArray> {
         let filter = filter.borrow();
-        let inner =
-            vortex::compute::filter(&self.inner, FilterMask::try_from(filter.inner.clone())?)?;
+        let inner = vortex::compute::filter(&self.inner, &Mask::try_from(filter.inner.clone())?)?;
         Ok(PyArray { inner })
     }
 
@@ -414,7 +414,7 @@ impl PyArray {
     ///     >>> a = vortex.array(['a', 'b', 'c', 'd'])
     ///     >>> indices = vortex.array([0, 2])
     ///     >>> a.take(indices).to_arrow_array()
-    ///     <pyarrow.lib.StringViewArray object at ...>
+    ///     <pyarrow.lib.StringArray object at ...>
     ///     [
     ///       "a",
     ///       "c"
@@ -425,7 +425,7 @@ impl PyArray {
     ///     >>> a = vortex.array(['a', 'b', 'c', 'd'])
     ///     >>> indices = vortex.array([0, 1, 1, 0])
     ///     >>> a.take(indices).to_arrow_array()
-    ///     <pyarrow.lib.StringViewArray object at ...>
+    ///     <pyarrow.lib.StringArray object at ...>
     ///     [
     ///       "a",
     ///       "b",
@@ -442,7 +442,7 @@ impl PyArray {
             )));
         }
 
-        let inner = take(&self.inner, indices, TakeOptions::default())?;
+        let inner = take(&self.inner, indices)?;
         Ok(PyArray { inner })
     }
 
@@ -467,7 +467,7 @@ impl PyArray {
     ///
     ///     >>> a = vortex.array(['a', 'b', 'c', 'd'])
     ///     >>> a.slice(1, 3).to_arrow_array()
-    ///     <pyarrow.lib.StringViewArray object at ...>
+    ///     <pyarrow.lib.StringArray object at ...>
     ///     [
     ///       "b",
     ///       "c"
@@ -477,7 +477,7 @@ impl PyArray {
     ///
     ///     >>> a = vortex.array(['a', 'b', 'c', 'd'])
     ///     >>> a.slice(3, 3).to_arrow_array()
-    ///     <pyarrow.lib.StringViewArray object at ...>
+    ///     <pyarrow.lib.StringArray object at ...>
     ///     []
     ///
     /// Unlike Python, it is an error to slice outside the bounds of the array:
@@ -519,12 +519,12 @@ impl PyArray {
     ///
     ///     >>> arr = vortex.array([1, 2, None, 3])
     ///     >>> print(arr.tree_display())
-    ///     root: vortex.primitive(0x03)(i64?, len=4) nbytes=33 B (100.00%)
+    ///     root: vortex.primitive(0x03)(i64?, len=4) nbytes=36 B (100.00%)
     ///       metadata: PrimitiveMetadata { validity: Array }
-    ///       buffer: 32 B
-    ///       validity: vortex.bool(0x02)(bool, len=4) nbytes=1 B (3.03%)
+    ///       buffer (align=8): 32 B
+    ///       validity: vortex.bool(0x02)(bool, len=4) nbytes=3 B (8.33%)
     ///         metadata: BoolMetadata { validity: NonNullable, first_byte_bit_offset: 0 }
-    ///         buffer: 1 B
+    ///         buffer (align=1): 1 B
     ///     <BLANKLINE>
     ///
     /// Compressed arrays often have more complex, deeply nested encoding trees.
