@@ -2,8 +2,7 @@ use vortex_error::VortexResult;
 use vortex_scalar::Scalar;
 
 use crate::array::{BoolArray, BoolEncoding};
-use crate::compute::unary::ScalarAtFn;
-use crate::ArrayDType;
+use crate::compute::ScalarAtFn;
 
 impl ScalarAtFn<BoolArray> for BoolEncoding {
     fn scalar_at(&self, array: &BoolArray, index: usize) -> VortexResult<Scalar> {
