@@ -1,36 +1,50 @@
 use std::fmt::{Debug, Display};
-use std::sync::Arc;
 
 use arrow_array::BooleanArray;
-use arrow_buffer::{BooleanBufferBuilder, MutableBuffer};
-use itertools::Itertools;
-use num_traits::AsPrimitive;
-use serde::{Deserialize, Serialize};
-use vortex_buffer::Buffer;
+use arrow_buffer::MutableBuffer;
+use vortex_buffer::{Alignment, ByteBuffer};
 use vortex_dtype::{DType, Nullability};
 use vortex_error::{vortex_bail, VortexExpect as _, VortexResult};
 
-use crate::encoding::ids;
+use crate::encoding::encoding_ids;
 use crate::stats::StatsSet;
-use crate::validity::{LogicalValidity, Validity, ValidityMetadata, ValidityVTable};
-use crate::variants::{ArrayVariants, BoolArrayTrait};
-use crate::visitor::{ArrayVisitor, VisitorVTable};
-use crate::{
-    impl_encoding, ArrayData, ArrayLen, ArrayTrait, Canonical, IntoArrayData, IntoCanonical,
-};
+use crate::validity::{Validity, ValidityMetadata};
+use crate::variants::BoolArrayTrait;
+use crate::visitor::ArrayVisitor;
+use crate::vtable::{CanonicalVTable, ValidateVTable};
+use crate::{impl_encoding, Canonical, IntoArray, RkyvMetadata};
 
 pub mod compute;
+mod patch;
 mod stats;
 
 // Re-export the BooleanBuffer type on our API surface.
-pub use arrow_buffer::BooleanBuffer;
+pub use arrow_buffer::{BooleanBuffer, BooleanBufferBuilder};
+use vortex_mask::Mask;
 
-impl_encoding!("vortex.bool", ids::BOOL, Bool);
+use crate::vtable::{ValidityVTable, VariantsVTable, VisitorVTable};
 
-#[derive(Clone, Debug, Serialize, Deserialize)]
+impl_encoding!(
+    "vortex.bool",
+    encoding_ids::BOOL,
+    Bool,
+    RkyvMetadata<BoolMetadata>
+);
+
+#[derive(
+    Clone,
+    Debug,
+    rkyv::Archive,
+    rkyv::Portable,
+    rkyv::Serialize,
+    rkyv::Deserialize,
+    rkyv::bytecheck::CheckBytes,
+)]
+#[bytecheck(crate = rkyv::bytecheck)]
+#[repr(C)]
 pub struct BoolMetadata {
-    validity: ValidityMetadata,
-    first_byte_bit_offset: u8,
+    pub(crate) validity: ValidityMetadata,
+    pub(crate) first_byte_bit_offset: u8,
 }
 
 impl Display for BoolMetadata {
@@ -41,23 +55,23 @@ impl Display for BoolMetadata {
 
 impl BoolArray {
     /// Access internal array buffer
-    pub fn buffer(&self) -> &Buffer {
+    pub fn buffer(&self) -> &ByteBuffer {
         self.as_ref()
-            .buffer()
+            .byte_buffer(0)
             .vortex_expect("Missing buffer in BoolArray")
     }
 
     /// Convert array into its internal buffer
-    pub fn into_buffer(self) -> Buffer {
+    pub fn into_buffer(self) -> ByteBuffer {
         self.into_array()
-            .into_buffer()
+            .into_byte_buffer(0)
             .vortex_expect("BoolArray must have a buffer")
     }
 
     /// Get array values as an arrow [BooleanBuffer]
     pub fn boolean_buffer(&self) -> BooleanBuffer {
         BooleanBuffer::new(
-            self.buffer().clone().into_arrow(),
+            self.buffer().clone().into_arrow_buffer(),
             self.metadata().first_byte_bit_offset as usize,
             self.len(),
         )
@@ -72,7 +86,7 @@ impl BoolArray {
     pub fn into_boolean_builder(self) -> (BooleanBufferBuilder, usize) {
         let first_byte_bit_offset = self.metadata().first_byte_bit_offset as usize;
         let len = self.len();
-        let arrow_buffer = self.into_buffer().into_arrow();
+        let arrow_buffer = self.into_buffer().into_arrow_buffer();
         let mutable_buf = if arrow_buffer.ptr_offset() == 0 {
             arrow_buffer.into_mutable().unwrap_or_else(|b| {
                 let mut buf = MutableBuffer::with_capacity(b.len());
@@ -109,6 +123,7 @@ impl BoolArray {
 
     /// Create a new BoolArray from a buffer and validity metadata.
     /// Returns an error if the validity length does not match the buffer length.
+    #[allow(clippy::cast_possible_truncation)]
     pub fn try_new(buffer: BooleanBuffer, validity: Validity) -> VortexResult<Self> {
         let buffer_len = buffer.len();
         let buffer_offset = buffer.offset();
@@ -119,47 +134,17 @@ impl BoolArray {
             .into_inner()
             .bit_slice(buffer_byte_offset, buffer_len);
 
-        ArrayData::try_new_owned(
-            &BoolEncoding,
+        Self::try_from_parts(
             DType::Bool(validity.nullability()),
             buffer_len,
-            Arc::new(BoolMetadata {
+            RkyvMetadata(BoolMetadata {
                 validity: validity.to_metadata(buffer_len)?,
                 first_byte_bit_offset,
             }),
-            Some(Buffer::from(inner)),
-            validity.into_array().into_iter().collect_vec().into(),
+            Some(vec![ByteBuffer::from_arrow_buffer(inner, Alignment::of::<u8>())].into()),
+            validity.into_array().map(|v| [v].into()),
             StatsSet::default(),
-        )?
-        .try_into()
-    }
-
-    pub fn patch<P: AsPrimitive<usize>>(
-        self,
-        positions: &[P],
-        values: BoolArray,
-    ) -> VortexResult<Self> {
-        if positions.len() != values.len() {
-            vortex_bail!(
-                "Positions and values passed to patch had different lengths {} and {}",
-                positions.len(),
-                values.len()
-            );
-        }
-        if let Some(last_pos) = positions.last() {
-            if last_pos.as_() >= self.len() {
-                vortex_bail!(OutOfBounds: last_pos.as_(), 0, self.len())
-            }
-        }
-
-        let len = self.len();
-        let result_validity = self.validity().patch(len, positions, values.validity())?;
-        let (mut own_values, bit_offset) = self.into_boolean_builder();
-        for (idx, value) in positions.iter().zip_eq(values.boolean_buffer().iter()) {
-            own_values.set_bit(idx.as_() + bit_offset, value);
-        }
-
-        Self::try_new(own_values.finish().slice(bit_offset, len), result_validity)
+        )
     }
 
     /// Create a new BoolArray from a set of indices and a length.
@@ -176,20 +161,27 @@ impl BoolArray {
     }
 }
 
-impl ArrayTrait for BoolArray {}
+impl ValidateVTable<BoolArray> for BoolEncoding {
+    fn validate(&self, array: &BoolArray) -> VortexResult<()> {
+        if array.as_ref().nbuffers() != 1 {
+            vortex_bail!(
+                "BoolArray: expected 1 buffer, found {}",
+                array.as_ref().nbuffers()
+            );
+        }
 
-impl ArrayVariants for BoolArray {
-    fn as_bool_array(&self) -> Option<&dyn BoolArrayTrait> {
-        Some(self)
+        Ok(())
     }
 }
 
-impl BoolArrayTrait for BoolArray {
-    fn invert(&self) -> VortexResult<ArrayData> {
-        Ok(BoolArray::try_new(!&self.boolean_buffer(), self.validity())?.into_array())
+impl VariantsVTable<BoolArray> for BoolEncoding {
+    fn as_bool_array<'a>(&self, array: &'a BoolArray) -> Option<&'a dyn BoolArrayTrait> {
+        Some(array)
     }
 }
 
+impl BoolArrayTrait for BoolArray {}
+
 impl From<BooleanBuffer> for BoolArray {
     fn from(value: BooleanBuffer) -> Self {
         Self::new(value, Nullability::NonNullable)
@@ -213,18 +205,18 @@ impl FromIterator<Option<bool>> for BoolArray {
     }
 }
 
-impl IntoCanonical for BoolArray {
-    fn into_canonical(self) -> VortexResult<Canonical> {
-        Ok(Canonical::Bool(self))
+impl CanonicalVTable<BoolArray> for BoolEncoding {
+    fn into_canonical(&self, array: BoolArray) -> VortexResult<Canonical> {
+        Ok(Canonical::Bool(array))
     }
 }
 
 impl ValidityVTable<BoolArray> for BoolEncoding {
-    fn is_valid(&self, array: &BoolArray, index: usize) -> bool {
+    fn is_valid(&self, array: &BoolArray, index: usize) -> VortexResult<bool> {
         array.validity().is_valid(index)
     }
 
-    fn logical_validity(&self, array: &BoolArray) -> LogicalValidity {
+    fn logical_validity(&self, array: &BoolArray) -> VortexResult<Mask> {
         array.validity().to_logical(array.len())
     }
 }
@@ -238,13 +230,15 @@ impl VisitorVTable<BoolArray> for BoolEncoding {
 
 #[cfg(test)]
 mod tests {
-    use arrow_buffer::BooleanBuffer;
+    use arrow_buffer::{BooleanBuffer, BooleanBufferBuilder};
+    use vortex_buffer::buffer;
+    use vortex_dtype::Nullability;
 
-    use crate::array::BoolArray;
-    use crate::compute::slice;
-    use crate::compute::unary::scalar_at;
+    use crate::array::{BoolArray, PrimitiveArray};
+    use crate::compute::{scalar_at, slice};
+    use crate::patches::Patches;
     use crate::validity::Validity;
-    use crate::{IntoArrayData, IntoArrayVariant};
+    use crate::{IntoArray, IntoArrayVariant};
 
     #[test]
     fn bool_array() {
@@ -290,28 +284,52 @@ mod tests {
 
     #[test]
     fn patch_sliced_bools() {
-        let arr = BoolArray::from(BooleanBuffer::new_set(12));
-        let sliced = slice(arr, 4, 12).unwrap();
-        let (values, offset) = sliced.into_bool().unwrap().into_boolean_builder();
+        let arr = {
+            let mut builder = BooleanBufferBuilder::new(12);
+            builder.append(false);
+            builder.append_n(11, true);
+            BoolArray::from(builder.finish())
+        };
+        let sliced = slice(arr.clone(), 4, 12).unwrap();
+        let (values, offset) = sliced.clone().into_bool().unwrap().into_boolean_builder();
         assert_eq!(offset, 4);
-        assert_eq!(values.as_slice(), &[255, 15]);
-    }
+        assert_eq!(values.as_slice(), &[254, 15]);
 
-    #[test]
-    fn patch_sliced_bools_offset() {
-        let arr = BoolArray::from(BooleanBuffer::new_set(15));
-        let sliced = slice(arr, 4, 15).unwrap();
-        let (values, offset) = sliced.into_bool().unwrap().into_boolean_builder();
-        assert_eq!(offset, 4);
-        assert_eq!(values.as_slice(), &[255, 127]);
-    }
-
-    #[test]
-    fn patch_sliced_bools_even() {
-        let arr = BoolArray::from(BooleanBuffer::new_set(31));
-        let sliced = slice(arr, 8, 24).unwrap();
-        let (values, offset) = sliced.into_bool().unwrap().into_boolean_builder();
+        // patch the underlying array
+        let patches = Patches::new(
+            arr.len(),
+            PrimitiveArray::new(buffer![4u32], Validity::AllValid).into_array(),
+            BoolArray::from(BooleanBuffer::new_unset(1)).into_array(),
+        );
+        let arr = arr.patch(patches).unwrap();
+        let (values, offset) = arr.into_bool().unwrap().into_boolean_builder();
         assert_eq!(offset, 0);
-        assert_eq!(values.as_slice(), &[255, 255]);
+        assert_eq!(values.as_slice(), &[238, 15]);
+
+        // the slice should be unchanged
+        let (values, offset) = sliced.into_bool().unwrap().into_boolean_builder();
+        assert_eq!(offset, 4);
+        assert_eq!(values.as_slice(), &[254, 15]); // unchanged
+    }
+
+    #[test]
+    #[should_panic]
+    fn patch_bools_owned() {
+        let buffer = buffer![255u8; 2];
+        let buf = BooleanBuffer::new(buffer.into_arrow_buffer(), 0, 15);
+        let arr = BoolArray::new(buf, Nullability::NonNullable);
+        let buf_ptr = arr.boolean_buffer().sliced().as_ptr();
+
+        let patches = Patches::new(
+            arr.len(),
+            PrimitiveArray::new(buffer![0u32], Validity::AllValid).into_array(),
+            BoolArray::from(BooleanBuffer::new_unset(1)).into_array(),
+        );
+        let arr = arr.patch(patches).unwrap();
+        assert_eq!(arr.boolean_buffer().sliced().as_ptr(), buf_ptr);
+
+        let (values, offset) = arr.into_bool().unwrap().into_boolean_builder();
+        assert_eq!(offset, 0);
+        assert_eq!(values.as_slice(), &[254, 127]);
     }
 }
