@@ -7,34 +7,40 @@ use std::fmt::{Debug, Display};
 use futures_util::stream;
 use itertools::Itertools;
 use serde::{Deserialize, Serialize};
+use vortex_buffer::BufferMut;
 use vortex_dtype::{DType, Nullability, PType};
-use vortex_error::{vortex_bail, vortex_panic, VortexExpect as _, VortexResult};
-use vortex_scalar::Scalar;
+use vortex_error::{vortex_bail, vortex_panic, VortexExpect as _, VortexResult, VortexUnwrap};
+use vortex_mask::Mask;
 
 use crate::array::primitive::PrimitiveArray;
-use crate::compute::unary::{scalar_at, subtract_scalar, SubtractScalarFn};
-use crate::compute::{search_sorted, SearchSortedSide};
-use crate::encoding::ids;
+use crate::compute::{scalar_at, search_sorted_usize, SearchSortedSide};
+use crate::encoding::encoding_ids;
 use crate::iter::{ArrayIterator, ArrayIteratorAdapter};
-use crate::stats::ArrayStatistics;
+use crate::stats::StatsSet;
 use crate::stream::{ArrayStream, ArrayStreamAdapter};
+use crate::validity::Validity;
 use crate::validity::Validity::NonNullable;
-use crate::validity::{LogicalValidity, Validity, ValidityVTable};
-use crate::visitor::{ArrayVisitor, VisitorVTable};
-use crate::{
-    impl_encoding, ArrayDType, ArrayData, ArrayLen, ArrayTrait, IntoArrayData, IntoCanonical,
-};
+use crate::visitor::ArrayVisitor;
+use crate::vtable::{ValidateVTable, ValidityVTable, VisitorVTable};
+use crate::{impl_encoding, Array, IntoArray, IntoCanonical, RkyvMetadata};
 
 mod canonical;
 mod compute;
 mod stats;
 mod variants;
 
-impl_encoding!("vortex.chunked", ids::CHUNKED, Chunked);
+impl_encoding!(
+    "vortex.chunked",
+    encoding_ids::CHUNKED,
+    Chunked,
+    RkyvMetadata<ChunkedMetadata>
+);
 
-#[derive(Clone, Debug, Serialize, Deserialize)]
+#[derive(
+    Clone, Debug, Serialize, Deserialize, rkyv::Archive, rkyv::Serialize, rkyv::Deserialize,
+)]
 pub struct ChunkedMetadata {
-    nchunks: usize,
+    pub(crate) nchunks: usize,
 }
 
 impl Display for ChunkedMetadata {
@@ -46,58 +52,52 @@ impl Display for ChunkedMetadata {
 impl ChunkedArray {
     const ENDS_DTYPE: DType = DType::Primitive(PType::U64, Nullability::NonNullable);
 
-    pub fn try_new(chunks: Vec<ArrayData>, dtype: DType) -> VortexResult<Self> {
+    pub fn try_new(chunks: Vec<Array>, dtype: DType) -> VortexResult<Self> {
         for chunk in &chunks {
             if chunk.dtype() != &dtype {
                 vortex_bail!(MismatchedTypes: dtype, chunk.dtype());
             }
         }
 
-        let chunk_offsets = [0u64]
-            .into_iter()
-            .chain(chunks.iter().map(|c| c.len() as u64))
-            .scan(0, |acc, c| {
-                *acc += c;
-                Some(*acc)
-            })
-            .collect_vec();
+        let mut chunk_offsets = BufferMut::<u64>::with_capacity(chunks.len() + 1);
+        chunk_offsets.extend(
+            [0u64]
+                .into_iter()
+                .chain(chunks.iter().map(|c| c.len() as u64))
+                .scan(0, |acc, c| {
+                    *acc += c;
+                    Some(*acc)
+                }),
+        );
 
         let nchunks = chunk_offsets.len() - 1;
         let length = *chunk_offsets
             .last()
-            .vortex_expect("Chunk ends is guaranteed to have at least one element")
-            as usize;
-
-        let stats = chunks
-            .iter()
-            .map(|chunk| chunk.statistics().to_set())
-            .reduce(|mut acc, stats| {
-                acc.merge_ordered(&stats);
-                acc
-            })
-            .unwrap_or_default();
+            .vortex_expect("Chunk ends is guaranteed to have at least one element");
 
         let mut children = Vec::with_capacity(chunks.len() + 1);
-        children.push(PrimitiveArray::from_vec(chunk_offsets, NonNullable).into_array());
+        children.push(PrimitiveArray::new(chunk_offsets, NonNullable).into_array());
         children.extend(chunks);
 
         Self::try_from_parts(
             dtype,
-            length,
-            ChunkedMetadata { nchunks },
-            children.into(),
-            stats,
+            length.try_into().vortex_unwrap(),
+            RkyvMetadata(ChunkedMetadata { nchunks }),
+            None,
+            Some(children.into()),
+            StatsSet::default(),
         )
     }
 
     #[inline]
-    pub fn chunk(&self, idx: usize) -> VortexResult<ArrayData> {
+    pub fn chunk(&self, idx: usize) -> VortexResult<Array> {
         if idx >= self.nchunks() {
             vortex_bail!("chunk index {} > num chunks ({})", idx, self.nchunks());
         }
 
-        let chunk_start = usize::try_from(&scalar_at(self.chunk_offsets(), idx)?)?;
-        let chunk_end = usize::try_from(&scalar_at(self.chunk_offsets(), idx + 1)?)?;
+        let chunk_offsets = self.chunk_offsets();
+        let chunk_start = usize::try_from(&scalar_at(&chunk_offsets, idx)?)?;
+        let chunk_end = usize::try_from(&scalar_at(&chunk_offsets, idx + 1)?)?;
 
         // Offset the index since chunk_ends is child 0.
         self.as_ref()
@@ -109,7 +109,7 @@ impl ChunkedArray {
     }
 
     #[inline]
-    pub fn chunk_offsets(&self) -> ArrayData {
+    pub fn chunk_offsets(&self) -> Array {
         self.as_ref()
             .child(0, &Self::ENDS_DTYPE, self.nchunks() + 1)
             .vortex_expect("Missing chunk ends in ChunkedArray")
@@ -120,10 +120,11 @@ impl ChunkedArray {
 
         // Since there might be duplicate values in offsets because of empty chunks we want to search from right
         // and take the last chunk (we subtract 1 since there's a leading 0)
-        let index_chunk = search_sorted(&self.chunk_offsets(), index, SearchSortedSide::Right)
-            .vortex_expect("Search sorted failed in find_chunk_idx")
-            .to_ends_index(self.nchunks() + 1)
-            .saturating_sub(1);
+        let index_chunk =
+            search_sorted_usize(&self.chunk_offsets(), index, SearchSortedSide::Right)
+                .vortex_expect("Search sorted failed in find_chunk_idx")
+                .to_ends_index(self.nchunks() + 1)
+                .saturating_sub(1);
         let chunk_start = scalar_at(self.chunk_offsets(), index_chunk)
             .and_then(|s| usize::try_from(&s))
             .vortex_expect("Failed to find chunk start in find_chunk_idx");
@@ -132,12 +133,12 @@ impl ChunkedArray {
         (index_chunk, index_in_chunk)
     }
 
-    pub fn chunks(&self) -> impl Iterator<Item = ArrayData> + '_ {
+    pub fn chunks(&self) -> impl Iterator<Item = Array> + '_ {
         (0..self.nchunks()).map(|c| {
             self.chunk(c).unwrap_or_else(|e| {
                 vortex_panic!(
                     e,
-                    "ChunkedArray: chunks: chunk {} should exist (nchunks: {})",
+                    "ChunkedArray: chunks: chunk {} should exist (num chunks: {})",
                     c,
                     self.nchunks()
                 )
@@ -189,6 +190,7 @@ impl ChunkedArray {
         if !chunks_to_combine.is_empty() {
             new_chunks.push(
                 ChunkedArray::try_new(chunks_to_combine, self.dtype().clone())?
+                    .into_array()
                     .into_canonical()?
                     .into(),
             );
@@ -198,11 +200,11 @@ impl ChunkedArray {
     }
 }
 
-impl ArrayTrait for ChunkedArray {}
+impl ValidateVTable<ChunkedArray> for ChunkedEncoding {}
 
-impl FromIterator<ArrayData> for ChunkedArray {
-    fn from_iter<T: IntoIterator<Item = ArrayData>>(iter: T) -> Self {
-        let chunks: Vec<ArrayData> = iter.into_iter().collect();
+impl FromIterator<Array> for ChunkedArray {
+    fn from_iter<T: IntoIterator<Item = Array>>(iter: T) -> Self {
+        let chunks: Vec<Array> = iter.into_iter().collect();
         let dtype = chunks
             .first()
             .map(|c| c.dtype().clone())
@@ -215,61 +217,42 @@ impl VisitorVTable<ChunkedArray> for ChunkedEncoding {
     fn accept(&self, array: &ChunkedArray, visitor: &mut dyn ArrayVisitor) -> VortexResult<()> {
         visitor.visit_child("chunk_ends", &array.chunk_offsets())?;
         for (idx, chunk) in array.chunks().enumerate() {
-            visitor.visit_child(format!("[{}]", idx).as_str(), &chunk)?;
+            visitor.visit_child(format!("chunks[{}]", idx).as_str(), &chunk)?;
         }
         Ok(())
     }
 }
 
 impl ValidityVTable<ChunkedArray> for ChunkedEncoding {
-    fn is_valid(&self, array: &ChunkedArray, index: usize) -> bool {
+    fn is_valid(&self, array: &ChunkedArray, index: usize) -> VortexResult<bool> {
         let (chunk, offset_in_chunk) = array.find_chunk_idx(index);
-        array
-            .chunk(chunk)
-            .unwrap_or_else(|e| {
-                vortex_panic!(e, "ChunkedArray: is_valid failed to find chunk {}", index)
-            })
-            .with_dyn(|a| a.is_valid(offset_in_chunk))
+        array.chunk(chunk)?.is_valid(offset_in_chunk)
     }
 
-    fn logical_validity(&self, array: &ChunkedArray) -> LogicalValidity {
-        let validity = array
-            .chunks()
-            .map(|a| a.with_dyn(|arr| arr.logical_validity()))
-            .collect::<Validity>();
+    fn logical_validity(&self, array: &ChunkedArray) -> VortexResult<Mask> {
+        // TODO(ngates): implement FromIterator<LogicalValidity> for LogicalValidity.
+        let validity: Validity = array.chunks().map(|a| a.logical_validity()).try_collect()?;
         validity.to_logical(array.len())
     }
 }
 
-impl SubtractScalarFn<ChunkedArray> for ChunkedEncoding {
-    fn subtract_scalar(
-        &self,
-        array: &ChunkedArray,
-        to_subtract: &Scalar,
-    ) -> VortexResult<ArrayData> {
-        let chunks = array
-            .chunks()
-            .map(|chunk| subtract_scalar(&chunk, to_subtract))
-            .collect::<VortexResult<Vec<_>>>()?;
-        Ok(ChunkedArray::try_new(chunks, array.dtype().clone())?.into_array())
-    }
-}
-
 #[cfg(test)]
 mod test {
+    use vortex_buffer::buffer;
     use vortex_dtype::{DType, Nullability, PType};
     use vortex_error::VortexResult;
 
     use crate::array::chunked::ChunkedArray;
-    use crate::compute::unary::{scalar_at, subtract_scalar};
-    use crate::{assert_arrays_eq, ArrayDType, IntoArrayData, IntoArrayVariant};
+    use crate::compute::test_harness::test_binary_numeric;
+    use crate::compute::{scalar_at, sub_scalar, try_cast};
+    use crate::{assert_arrays_eq, IntoArray, IntoArrayVariant};
 
     fn chunked_array() -> ChunkedArray {
         ChunkedArray::try_new(
             vec![
-                vec![1u64, 2, 3].into_array(),
-                vec![4u64, 5, 6].into_array(),
-                vec![7u64, 8, 9].into_array(),
+                buffer![1u64, 2, 3].into_array(),
+                buffer![4u64, 5, 6].into_array(),
+                buffer![7u64, 8, 9].into_array(),
             ],
             DType::Primitive(PType::U64, Nullability::NonNullable),
         )
@@ -278,9 +261,9 @@ mod test {
 
     #[test]
     fn test_scalar_subtract() {
-        let chunked = chunked_array();
+        let chunked = chunked_array().into_array();
         let to_subtract = 1u64;
-        let array = subtract_scalar(&chunked, &to_subtract.into()).unwrap();
+        let array = sub_scalar(&chunked, to_subtract.into()).unwrap();
 
         let chunked = ChunkedArray::try_from(array).unwrap();
         let mut chunks_out = chunked.chunks();
@@ -290,7 +273,7 @@ mod test {
             .unwrap()
             .into_primitive()
             .unwrap()
-            .maybe_null_slice::<u64>()
+            .as_slice::<u64>()
             .to_vec();
         assert_eq!(results, &[0u64, 1, 2]);
         let results = chunks_out
@@ -298,7 +281,7 @@ mod test {
             .unwrap()
             .into_primitive()
             .unwrap()
-            .maybe_null_slice::<u64>()
+            .as_slice::<u64>()
             .to_vec();
         assert_eq!(results, &[3u64, 4, 5]);
         let results = chunks_out
@@ -306,7 +289,7 @@ mod test {
             .unwrap()
             .into_primitive()
             .unwrap()
-            .maybe_null_slice::<u64>()
+            .as_slice::<u64>()
             .to_vec();
         assert_eq!(results, &[6u64, 7, 8]);
     }
@@ -314,7 +297,7 @@ mod test {
     #[test]
     fn test_rechunk_one_chunk() {
         let chunked = ChunkedArray::try_new(
-            vec![vec![0u64].into_array()],
+            vec![buffer![0u64].into_array()],
             DType::Primitive(PType::U64, Nullability::NonNullable),
         )
         .unwrap();
@@ -327,7 +310,7 @@ mod test {
     #[test]
     fn test_rechunk_two_chunks() {
         let chunked = ChunkedArray::try_new(
-            vec![vec![0u64].into_array(), vec![5u64].into_array()],
+            vec![buffer![0u64].into_array(), buffer![5u64].into_array()],
             DType::Primitive(PType::U64, Nullability::NonNullable),
         )
         .unwrap();
@@ -341,7 +324,10 @@ mod test {
     #[test]
     fn test_rechunk_tiny_target_chunks() {
         let chunked = ChunkedArray::try_new(
-            vec![vec![0u64, 1, 2, 3].into_array(), vec![4u64, 5].into_array()],
+            vec![
+                buffer![0u64, 1, 2, 3].into_array(),
+                buffer![4u64, 5].into_array(),
+            ],
             DType::Primitive(PType::U64, Nullability::NonNullable),
         )
         .unwrap();
@@ -357,11 +343,11 @@ mod test {
     fn test_rechunk_with_too_big_chunk() {
         let chunked = ChunkedArray::try_new(
             vec![
-                vec![0u64, 1, 2].into_array(),
-                vec![42_u64; 6].into_array(),
-                vec![4u64, 5].into_array(),
-                vec![6u64, 7].into_array(),
-                vec![8u64, 9].into_array(),
+                buffer![0u64, 1, 2].into_array(),
+                buffer![42_u64; 6].into_array(),
+                buffer![4u64, 5].into_array(),
+                buffer![6u64, 7].into_array(),
+                buffer![8u64, 9].into_array(),
             ],
             DType::Primitive(PType::U64, Nullability::NonNullable),
         )
@@ -373,4 +359,13 @@ mod test {
         assert_eq!(rechunked.nchunks(), 4);
         assert_arrays_eq!(chunked, rechunked);
     }
+
+    #[test]
+    fn test_chunked_binary_numeric() {
+        let array = chunked_array().into_array();
+        // The tests test both X - 1 and 1 - X, so we need signed values
+        let signed_dtype = DType::from(PType::try_from(array.dtype()).unwrap().to_signed());
+        let array = try_cast(array, &signed_dtype).unwrap();
+        test_binary_numeric::<u64>(array)
+    }
 }
