@@ -2,7 +2,8 @@ use vortex_error::VortexResult;
 
 use crate::array::chunked::ChunkedArray;
 use crate::array::ChunkedEncoding;
-use crate::stats::{ArrayStatistics, Stat, StatisticsVTable, StatsSet};
+use crate::stats::{Stat, StatsSet};
+use crate::vtable::StatisticsVTable;
 
 impl StatisticsVTable<ChunkedArray> for ChunkedEncoding {
     fn compute_statistics(&self, array: &ChunkedArray, stat: Stat) -> VortexResult<StatsSet> {
@@ -12,13 +13,15 @@ impl StatisticsVTable<ChunkedArray> for ChunkedEncoding {
             .chunks()
             .map(|c| {
                 let s = c.statistics();
-                s.compute(stat);
-                s.to_set()
-            })
-            .reduce(|mut acc, x| {
-                acc.merge_ordered(&x);
-                acc
+                match stat {
+                    Stat::IsConstant | Stat::IsSorted | Stat::IsStrictSorted => {
+                        s.compute_all(&[stat, Stat::Min, Stat::Max]).ok()
+                    }
+                    _ => s.compute(stat).map(|s| StatsSet::of(stat, s)),
+                }
+                .unwrap_or_default()
             })
+            .reduce(|acc, x| acc.merge_ordered(&x, array.dtype()))
             .unwrap_or_default())
     }
 }
