@@ -2,15 +2,15 @@ use vortex_error::VortexResult;
 
 use crate::array::{ConstantArray, ConstantEncoding};
 use crate::compute::{scalar_cmp, CompareFn, Operator};
-use crate::{ArrayData, ArrayLen, IntoArrayData};
+use crate::{Array, IntoArray};
 
 impl CompareFn<ConstantArray> for ConstantEncoding {
     fn compare(
         &self,
         lhs: &ConstantArray,
-        rhs: &ArrayData,
+        rhs: &Array,
         operator: Operator,
-    ) -> VortexResult<Option<ArrayData>> {
+    ) -> VortexResult<Option<Array>> {
         // We only support comparing a constant array to another constant array.
         // For all other encodings, we assume the constant is on the RHS.
         if let Some(const_scalar) = rhs.as_constant() {
