@@ -2,16 +2,15 @@ use vortex_error::VortexResult;
 
 use crate::array::{ConstantArray, ExtensionArray, ExtensionEncoding};
 use crate::compute::{compare, CompareFn, Operator};
-use crate::encoding::EncodingVTable;
-use crate::{ArrayData, ArrayLen};
+use crate::Array;
 
 impl CompareFn<ExtensionArray> for ExtensionEncoding {
     fn compare(
         &self,
         lhs: &ExtensionArray,
-        rhs: &ArrayData,
+        rhs: &Array,
         operator: Operator,
-    ) -> VortexResult<Option<ArrayData>> {
+    ) -> VortexResult<Option<Array>> {
         // If the RHS is a constant, we can extract the storage scalar.
         if let Some(const_ext) = rhs.as_constant() {
             let storage_scalar = const_ext.as_extension().storage();
@@ -24,8 +23,7 @@ impl CompareFn<ExtensionArray> for ExtensionEncoding {
         }
 
         // If the RHS is an extension array matching ours, we can extract the storage.
-        if rhs.is_encoding(ExtensionEncoding.id()) {
-            let rhs_ext = ExtensionArray::try_from(rhs.clone())?;
+        if let Some(rhs_ext) = ExtensionArray::maybe_from(rhs) {
             return compare(lhs.storage(), rhs_ext.storage(), operator).map(Some);
         }
 
