@@ -1,25 +1,30 @@
+use std::iter;
+
 use vortex_dtype::NativePType;
 use vortex_error::VortexResult;
 
 use crate::accessor::ArrayAccessor;
 use crate::array::primitive::PrimitiveArray;
-use crate::validity::ArrayValidity;
+use crate::validity::Validity;
+use crate::IntoArrayVariant;
 
 impl<T: NativePType> ArrayAccessor<T> for PrimitiveArray {
     fn with_iterator<F, R>(&self, f: F) -> VortexResult<R>
     where
         F: for<'a> FnOnce(&mut (dyn Iterator<Item = Option<&'a T>>)) -> R,
     {
-        match self.logical_validity().to_null_buffer()? {
-            None => {
-                let mut iter = self.maybe_null_slice::<T>().iter().map(Some);
+        match self.validity() {
+            Validity::NonNullable | Validity::AllValid => {
+                let mut iter = self.as_slice::<T>().iter().map(Some);
                 Ok(f(&mut iter))
             }
-            Some(nulls) => {
+            Validity::AllInvalid => Ok(f(&mut iter::repeat_n(None, self.len()))),
+            Validity::Array(v) => {
+                let validity = v.into_bool()?.boolean_buffer();
                 let mut iter = self
-                    .maybe_null_slice::<T>()
+                    .as_slice::<T>()
                     .iter()
-                    .zip(nulls.iter())
+                    .zip(validity.iter())
                     .map(|(value, valid)| valid.then_some(value));
                 Ok(f(&mut iter))
             }
