@@ -1,27 +1,37 @@
 use std::fmt::{Debug, Display};
+use std::sync::Arc;
 
 use serde::{Deserialize, Serialize};
-use vortex_dtype::field::Field;
-use vortex_dtype::{DType, FieldName, FieldNames, StructDType};
+use vortex_dtype::{DType, Field, FieldName, FieldNames, StructDType};
 use vortex_error::{vortex_bail, vortex_err, vortex_panic, VortexExpect as _, VortexResult};
+use vortex_mask::Mask;
 
-use crate::encoding::ids;
-use crate::stats::{ArrayStatistics, Stat, StatisticsVTable, StatsSet};
-use crate::validity::{LogicalValidity, Validity, ValidityMetadata, ValidityVTable};
-use crate::variants::{ArrayVariants, StructArrayTrait};
-use crate::visitor::{ArrayVisitor, VisitorVTable};
-use crate::{
-    impl_encoding, ArrayDType, ArrayData, ArrayLen, ArrayTrait, Canonical, IntoArrayData,
-    IntoCanonical,
+use crate::encoding::encoding_ids;
+use crate::stats::{Stat, StatsSet};
+use crate::validity::{Validity, ValidityMetadata};
+use crate::variants::StructArrayTrait;
+use crate::visitor::ArrayVisitor;
+use crate::vtable::{
+    CanonicalVTable, StatisticsVTable, ValidateVTable, ValidityVTable, VariantsVTable,
+    VisitorVTable,
 };
+use crate::{impl_encoding, Array, Canonical, IntoArray, RkyvMetadata};
 
 mod compute;
 
-impl_encoding!("vortex.struct", ids::STRUCT, Struct);
+impl_encoding!(
+    "vortex.struct",
+    encoding_ids::STRUCT,
+    Struct,
+    RkyvMetadata<StructMetadata>
+);
 
-#[derive(Clone, Debug, Serialize, Deserialize)]
+#[derive(
+    Clone, Debug, Serialize, Deserialize, rkyv::Archive, rkyv::Serialize, rkyv::Deserialize,
+)]
+#[repr(C)]
 pub struct StructMetadata {
-    validity: ValidityMetadata,
+    pub(crate) validity: ValidityMetadata,
 }
 
 impl Display for StructMetadata {
@@ -39,9 +49,9 @@ impl StructArray {
         })
     }
 
-    pub fn children(&self) -> impl Iterator<Item = ArrayData> + '_ {
+    pub fn children(&self) -> impl Iterator<Item = Array> + '_ {
         (0..self.nfields()).map(move |idx| {
-            self.field(idx).unwrap_or_else(|| {
+            self.maybe_null_field_by_idx(idx).unwrap_or_else(|| {
                 vortex_panic!("Field {} not found, nfields: {}", idx, self.nfields())
             })
         })
@@ -49,7 +59,7 @@ impl StructArray {
 
     pub fn try_new(
         names: FieldNames,
-        fields: Vec<ArrayData>,
+        fields: Vec<Array>,
         length: usize,
         validity: Validity,
     ) -> VortexResult<Self> {
@@ -79,28 +89,31 @@ impl StructArray {
         }
 
         Self::try_from_parts(
-            DType::Struct(StructDType::new(names, field_dtypes), nullability),
+            DType::Struct(Arc::new(StructDType::new(names, field_dtypes)), nullability),
             length,
-            StructMetadata {
+            RkyvMetadata(StructMetadata {
                 validity: validity_metadata,
-            },
-            children.into(),
+            }),
+            None,
+            Some(children.into()),
             StatsSet::default(),
         )
     }
 
-    pub fn from_fields<N: AsRef<str>>(items: &[(N, ArrayData)]) -> VortexResult<Self> {
-        let names: Vec<FieldName> = items
-            .iter()
-            .map(|(name, _)| FieldName::from(name.as_ref()))
-            .collect();
-        let fields: Vec<ArrayData> = items.iter().map(|(_, array)| array.clone()).collect();
+    pub fn from_fields<N: AsRef<str>>(items: &[(N, Array)]) -> VortexResult<Self> {
+        let names = items.iter().map(|(name, _)| FieldName::from(name.as_ref()));
+        let fields: Vec<Array> = items.iter().map(|(_, array)| array.clone()).collect();
         let len = fields
             .first()
             .map(|f| f.len())
             .ok_or_else(|| vortex_err!("StructArray cannot be constructed from an empty slice of arrays because the length is unspecified"))?;
 
-        Self::try_new(FieldNames::from(names), fields, len, Validity::NonNullable)
+        Self::try_new(
+            FieldNames::from_iter(names),
+            fields,
+            len,
+            Validity::NonNullable,
+        )
     }
 
     // TODO(aduffy): Add equivalent function to support field masks for nested column access.
@@ -111,24 +124,21 @@ impl StructArray {
     /// perform column re-ordering, deletion, or duplication at a logical level, without any data
     /// copying.
     #[allow(clippy::same_name_method)]
-    pub fn project(&self, projection: &[Field]) -> VortexResult<Self> {
+    pub fn project(&self, projection: &[FieldName]) -> VortexResult<Self> {
         let mut children = Vec::with_capacity(projection.len());
         let mut names = Vec::with_capacity(projection.len());
 
-        for field in projection.iter() {
-            let idx = match field {
-                Field::Name(n) => self
-                    .names()
-                    .iter()
-                    .position(|name| name.as_ref() == n)
-                    .ok_or_else(|| vortex_err!("Unknown field {n}"))?,
-                Field::Index(i) => *i,
-            };
+        for f_name in projection.iter() {
+            let idx = self
+                .names()
+                .iter()
+                .position(|name| name == f_name)
+                .ok_or_else(|| vortex_err!("Unknown field {f_name}"))?;
 
             names.push(self.names()[idx].clone());
             children.push(
-                self.field(idx)
-                    .ok_or_else(|| vortex_err!(OutOfBounds: idx, 0, self.dtypes().len()))?,
+                self.maybe_null_field_by_idx(idx)
+                    .ok_or_else(|| vortex_err!(OutOfBounds: idx, 0, self.nfields()))?,
             );
         }
 
@@ -141,41 +151,54 @@ impl StructArray {
     }
 }
 
-impl ArrayTrait for StructArray {}
+impl ValidateVTable<StructArray> for StructEncoding {}
 
-impl ArrayVariants for StructArray {
-    fn as_struct_array(&self) -> Option<&dyn StructArrayTrait> {
-        Some(self)
+impl VariantsVTable<StructArray> for StructEncoding {
+    fn as_struct_array<'a>(&self, array: &'a StructArray) -> Option<&'a dyn StructArrayTrait> {
+        Some(array)
     }
 }
 
 impl StructArrayTrait for StructArray {
-    fn field(&self, idx: usize) -> Option<ArrayData> {
-        self.dtypes().get(idx).map(|dtype| {
-            self.as_ref()
-                .child(idx, dtype, self.len())
-                .unwrap_or_else(|e| vortex_panic!(e, "StructArray: field {} not found", idx))
-        })
+    fn maybe_null_field_by_idx(&self, idx: usize) -> Option<Array> {
+        Some(
+            self.field_info(&Field::Index(idx))
+                .map(|field_info| {
+                    self.as_ref()
+                        .child(
+                            idx,
+                            &field_info
+                                .dtype
+                                .value()
+                                .vortex_expect("FieldInfo could not access dtype"),
+                            self.len(),
+                        )
+                        .unwrap_or_else(|e| {
+                            vortex_panic!(e, "StructArray: field {} not found", idx)
+                        })
+                })
+                .unwrap_or_else(|e| vortex_panic!(e, "StructArray: field {} not found", idx)),
+        )
     }
 
-    fn project(&self, projection: &[Field]) -> VortexResult<ArrayData> {
+    fn project(&self, projection: &[FieldName]) -> VortexResult<Array> {
         self.project(projection).map(|a| a.into_array())
     }
 }
 
-impl IntoCanonical for StructArray {
+impl CanonicalVTable<StructArray> for StructEncoding {
     /// StructEncoding is the canonical form for a [DType::Struct] array, so return self.
-    fn into_canonical(self) -> VortexResult<Canonical> {
-        Ok(Canonical::Struct(self))
+    fn into_canonical(&self, array: StructArray) -> VortexResult<Canonical> {
+        Ok(Canonical::Struct(array))
     }
 }
 
 impl ValidityVTable<StructArray> for StructEncoding {
-    fn is_valid(&self, array: &StructArray, index: usize) -> bool {
+    fn is_valid(&self, array: &StructArray, index: usize) -> VortexResult<bool> {
         array.validity().is_valid(index)
     }
 
-    fn logical_validity(&self, array: &StructArray) -> LogicalValidity {
+    fn logical_validity(&self, array: &StructArray) -> VortexResult<Mask> {
         array.validity().to_logical(array.len())
     }
 }
@@ -184,9 +207,9 @@ impl VisitorVTable<StructArray> for StructEncoding {
     fn accept(&self, array: &StructArray, visitor: &mut dyn ArrayVisitor) -> VortexResult<()> {
         for (idx, name) in array.names().iter().enumerate() {
             let child = array
-                .field(idx)
+                .maybe_null_field_by_idx(idx)
                 .ok_or_else(|| vortex_err!(OutOfBounds: idx, 0, array.nfields()))?;
-            visitor.visit_child(&format!("\"{}\"", name), &child)?;
+            visitor.visit_child(name.as_ref(), &child)?;
         }
         Ok(())
     }
@@ -210,7 +233,7 @@ impl StatisticsVTable<StructArray> for StructEncoding {
 
 #[cfg(test)]
 mod test {
-    use vortex_dtype::field::Field;
+    use vortex_buffer::buffer;
     use vortex_dtype::{DType, FieldName, FieldNames, Nullability};
 
     use crate::array::primitive::PrimitiveArray;
@@ -219,11 +242,11 @@ mod test {
     use crate::array::BoolArray;
     use crate::validity::Validity;
     use crate::variants::StructArrayTrait;
-    use crate::{ArrayLen, IntoArrayData};
+    use crate::IntoArray;
 
     #[test]
     fn test_project() {
-        let xs = PrimitiveArray::from_vec(vec![0i64, 1, 2, 3, 4], Validity::NonNullable);
+        let xs = PrimitiveArray::new(buffer![0i64, 1, 2, 3, 4], Validity::NonNullable);
         let ys = VarBinArray::from_vec(
             vec!["a", "b", "c", "d", "e"],
             DType::Utf8(Nullability::NonNullable),
@@ -239,7 +262,7 @@ mod test {
         .unwrap();
 
         let struct_b = struct_a
-            .project(&[Field::from(2usize), Field::from(0)])
+            .project(&[FieldName::from("zs"), FieldName::from("xs")])
             .unwrap();
         assert_eq!(
             struct_b.names().as_ref(),
@@ -248,13 +271,13 @@ mod test {
 
         assert_eq!(struct_b.len(), 5);
 
-        let bools = BoolArray::try_from(struct_b.field(0).unwrap()).unwrap();
+        let bools = BoolArray::try_from(struct_b.maybe_null_field_by_idx(0).unwrap()).unwrap();
         assert_eq!(
             bools.boolean_buffer().iter().collect::<Vec<_>>(),
             vec![true, true, true, false, false]
         );
 
-        let prims = PrimitiveArray::try_from(struct_b.field(1).unwrap()).unwrap();
-        assert_eq!(prims.maybe_null_slice::<i64>(), [0i64, 1, 2, 3, 4]);
+        let prims = PrimitiveArray::try_from(struct_b.maybe_null_field_by_idx(1).unwrap()).unwrap();
+        assert_eq!(prims.as_slice::<i64>(), [0i64, 1, 2, 3, 4]);
     }
 }
