@@ -1,17 +1,17 @@
 use vortex_error::VortexResult;
+use vortex_mask::Mask;
 
 use crate::array::varbin::VarBinArray;
 use crate::array::VarBinEncoding;
-use crate::validity::{LogicalValidity, ValidityVTable};
-use crate::visitor::{ArrayVisitor, VisitorVTable};
-use crate::ArrayLen;
+use crate::visitor::ArrayVisitor;
+use crate::vtable::{ValidityVTable, VisitorVTable};
 
 impl ValidityVTable<VarBinArray> for VarBinEncoding {
-    fn is_valid(&self, array: &VarBinArray, index: usize) -> bool {
+    fn is_valid(&self, array: &VarBinArray, index: usize) -> VortexResult<bool> {
         array.validity().is_valid(index)
     }
 
-    fn logical_validity(&self, array: &VarBinArray) -> LogicalValidity {
+    fn logical_validity(&self, array: &VarBinArray) -> VortexResult<Mask> {
         array.validity().to_logical(array.len())
     }
 }
@@ -19,7 +19,7 @@ impl ValidityVTable<VarBinArray> for VarBinEncoding {
 impl VisitorVTable<VarBinArray> for VarBinEncoding {
     fn accept(&self, array: &VarBinArray, visitor: &mut dyn ArrayVisitor) -> VortexResult<()> {
         visitor.visit_child("offsets", &array.offsets())?;
-        visitor.visit_child("bytes", &array.bytes())?;
+        visitor.visit_buffer(&array.bytes())?;
         visitor.visit_validity(&array.validity())
     }
 }
