@@ -2,28 +2,23 @@ use std::fmt::{Debug, Display, Formatter};
 use std::ops::Range;
 use std::sync::Arc;
 
-use ::serde::{Deserialize, Serialize};
 use arrow_array::builder::{BinaryViewBuilder, GenericByteViewBuilder, StringViewBuilder};
 use arrow_array::types::{BinaryViewType, ByteViewType, StringViewType};
 use arrow_array::{ArrayRef, BinaryViewArray, GenericByteViewArray, StringViewArray};
 use arrow_buffer::ScalarBuffer;
-use itertools::Itertools;
 use static_assertions::{assert_eq_align, assert_eq_size};
-use vortex_buffer::Buffer;
-use vortex_dtype::{DType, PType};
-use vortex_error::{vortex_bail, vortex_err, vortex_panic, VortexExpect, VortexResult};
+use vortex_buffer::{Alignment, Buffer, ByteBuffer};
+use vortex_dtype::DType;
+use vortex_error::{vortex_bail, vortex_panic, VortexExpect, VortexResult, VortexUnwrap};
+use vortex_mask::Mask;
 
 use crate::arrow::FromArrowArray;
-use crate::compute::slice;
-use crate::encoding::ids;
+use crate::encoding::encoding_ids;
 use crate::stats::StatsSet;
-use crate::validity::{ArrayValidity, LogicalValidity, Validity, ValidityMetadata, ValidityVTable};
-use crate::variants::PrimitiveArrayTrait;
-use crate::visitor::{ArrayVisitor, VisitorVTable};
-use crate::{
-    impl_encoding, ArrayDType, ArrayData, ArrayLen, ArrayTrait, Canonical, IntoArrayVariant,
-    IntoCanonical,
-};
+use crate::validity::{Validity, ValidityMetadata};
+use crate::visitor::ArrayVisitor;
+use crate::vtable::{CanonicalVTable, ValidateVTable, ValidityVTable, VisitorVTable};
+use crate::{impl_encoding, Array, Canonical, RkyvMetadata};
 
 mod accessor;
 mod compute;
@@ -45,7 +40,7 @@ impl Inlined {
             value.len()
         );
         let mut inlined = Self {
-            size: value.len() as u32,
+            size: value.len().try_into().vortex_unwrap(),
             data: [0u8; BinaryView::MAX_INLINED_SIZE],
         };
         inlined.data[..value.len()].copy_from_slice(value);
@@ -99,7 +94,7 @@ impl Ref {
 }
 
 #[derive(Clone, Copy)]
-#[repr(C, align(8))]
+#[repr(C, align(16))]
 pub union BinaryView {
     // Numeric representation. This is logically `u128`, but we split it into the high and low
     // bits to preserve the alignment.
@@ -115,7 +110,7 @@ pub union BinaryView {
 assert_eq_size!(BinaryView, [u8; 16]);
 assert_eq_size!(Inlined, [u8; 16]);
 assert_eq_size!(Ref, [u8; 16]);
-assert_eq_align!(BinaryView, u64);
+assert_eq_align!(BinaryView, u128);
 
 impl BinaryView {
     pub const MAX_INLINED_SIZE: usize = 12;
@@ -150,6 +145,7 @@ impl BinaryView {
     }
 
     #[inline]
+    #[allow(clippy::cast_possible_truncation)]
     pub fn is_inlined(&self) -> bool {
         self.len() <= (Self::MAX_INLINED_SIZE as u32)
     }
@@ -188,17 +184,10 @@ impl Debug for BinaryView {
     }
 }
 
-// reminder: views are 16 bytes with 8-byte alignment
-pub(crate) const VIEW_SIZE_BYTES: usize = size_of::<BinaryView>();
-
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, rkyv::Archive, rkyv::Serialize, rkyv::Deserialize)]
 pub struct VarBinViewMetadata {
     // Validity metadata
-    validity: ValidityMetadata,
-
-    // Length of each buffer. The buffers are primitive byte arrays containing the raw string/binary
-    // data referenced by views.
-    buffer_lens: Vec<u32>,
+    pub(crate) validity: ValidityMetadata,
 }
 
 impl Display for VarBinViewMetadata {
@@ -207,43 +196,22 @@ impl Display for VarBinViewMetadata {
     }
 }
 
-pub struct Buffers<'a> {
-    index: u32,
-    n_buffers: u32,
-    array: &'a VarBinViewArray,
-}
-
-impl Iterator for Buffers<'_> {
-    type Item = ArrayData;
-
-    fn next(&mut self) -> Option<Self::Item> {
-        if self.index >= self.n_buffers {
-            return None;
-        }
-
-        let bytes = self.array.buffer(self.index as usize);
-        self.index += 1;
-        Some(bytes)
-    }
-}
-
-impl_encoding!("vortex.varbinview", ids::VAR_BIN_VIEW, VarBinView);
+impl_encoding!(
+    "vortex.varbinview",
+    encoding_ids::VAR_BIN_VIEW,
+    VarBinView,
+    RkyvMetadata<VarBinViewMetadata>
+);
 
 impl VarBinViewArray {
     pub fn try_new(
-        views: ArrayData,
-        buffers: Vec<ArrayData>,
+        views: Buffer<BinaryView>,
+        buffers: Vec<ByteBuffer>,
         dtype: DType,
         validity: Validity,
     ) -> VortexResult<Self> {
-        if !matches!(views.dtype(), &DType::BYTES) {
-            vortex_bail!(MismatchedTypes: "views u8", views.dtype());
-        }
-
-        for d in buffers.iter() {
-            if !matches!(d.dtype(), &DType::BYTES) {
-                vortex_bail!(MismatchedTypes: "buffers u8", d.dtype());
-            }
+        if views.alignment() != Alignment::of::<BinaryView>() {
+            vortex_bail!("Views must be aligned to a 128 bits");
         }
 
         if !matches!(dtype, DType::Binary(_) | DType::Utf8(_)) {
@@ -254,79 +222,65 @@ impl VarBinViewArray {
             vortex_bail!("incorrect validity {:?}", validity);
         }
 
-        let num_views = views.len() / VIEW_SIZE_BYTES;
-        let buffer_lens: Vec<u32> = buffers
-            .iter()
-            .map(|buffer| -> VortexResult<u32> {
-                u32::try_from(buffer.len())
-                    .map_err(|e| vortex_err!("buffer must be within 32-bit range: {e}"))
-            })
-            .try_collect()?;
         let metadata = VarBinViewMetadata {
-            validity: validity.to_metadata(num_views)?,
-            buffer_lens,
+            validity: validity.to_metadata(views.len())?,
         };
 
-        let mut children = Vec::with_capacity(buffers.len() + 2);
-        children.push(views);
-        children.extend(buffers);
-        if let Some(a) = validity.into_array() {
-            children.push(a)
-        }
+        let array_len = views.len();
+
+        let mut array_buffers = Vec::with_capacity(buffers.len() + 1);
+        array_buffers.push(views.into_byte_buffer());
+        array_buffers.extend(buffers);
 
         Self::try_from_parts(
             dtype,
-            num_views,
-            metadata,
-            children.into(),
+            array_len,
+            RkyvMetadata(metadata),
+            Some(array_buffers.into()),
+            validity.into_array().map(|v| [v].into()),
             StatsSet::default(),
         )
     }
 
     /// Number of raw string data buffers held by this array.
-    pub fn buffer_count(&self) -> usize {
-        self.metadata().buffer_lens.len()
+    pub fn nbuffers(&self) -> usize {
+        self.0.nbuffers() - 1
     }
 
-    /// Access to the underlying `views` child array as a slice of [BinaryView] structures.
+    /// Access to the primitive views buffer.
     ///
-    /// This is useful for iteration over the values, as well as for applying filters that may
-    /// only require hitting the prefixes or inline strings.
-    pub fn binary_views(&self) -> VortexResult<Views> {
-        Ok(Views {
-            inner: self.views().into_primitive()?.into_buffer(),
-            idx: 0,
-            len: self.len(),
-        })
-    }
-
-    /// Retrieve a binary view at a specific index.
-    ///
-    /// If you will be calling this method many times, it's recommended that you instead use the
-    /// iterator provided by [`binary_views`][Self::binary_views].
-    pub fn view_at(&self, index: usize) -> VortexResult<BinaryView> {
-        let start = index * VIEW_SIZE_BYTES;
-        let stop = (index + 1) * VIEW_SIZE_BYTES;
-        let view_bytes = slice(self.views(), start, stop)?
-            .into_primitive()?
-            .into_buffer();
-
-        let mut le_bytes: [u8; 16] = [0u8; 16];
-        le_bytes.copy_from_slice(view_bytes.as_slice());
-
-        Ok(BinaryView { le_bytes })
-    }
-
-    /// Access to the primitive views array.
-    ///
-    /// Variable-sized binary view arrays contain a "view" child array, with 16-byte entries that
+    /// Variable-sized binary view buffer contain a "view" child array, with 16-byte entries that
     /// contain either a pointer into one of the array's owned `buffer`s OR an inlined copy of
     /// the string (if the string has 12 bytes or fewer).
     #[inline]
-    pub fn views(&self) -> ArrayData {
-        self.as_ref()
-            .child(0, &DType::BYTES, self.len() * VIEW_SIZE_BYTES)
-            .vortex_expect("VarBinViewArray: views child")
+    pub fn views(&self) -> Buffer<BinaryView> {
+        Buffer::from_byte_buffer(
+            self.0
+                .byte_buffer(0)
+                .vortex_expect("Expected a views buffer")
+                .clone(),
+        )
+    }
+
+    /// Access value bytes at a given index
+    ///
+    /// Will return a bytebuffer pointing to the underlying data without performing a copy
+    #[inline]
+    pub fn bytes_at(&self, index: usize) -> ByteBuffer {
+        let views = self.views();
+        let view = &views[index];
+        // Expect this to be the common case: strings > 12 bytes.
+        if !view.is_inlined() {
+            let view_ref = view.as_view();
+            self.buffer(view_ref.buffer_index() as usize)
+                .slice(view_ref.to_range())
+        } else {
+            // Return access to the range of bytes around it.
+            views
+                .clone()
+                .into_byte_buffer()
+                .slice_ref(view.as_inlined().value())
+        }
     }
 
     /// Access one of the backing data buffers.
@@ -336,45 +290,31 @@ impl VarBinViewArray {
     /// This method panics if the provided index is out of bounds for the set of buffers provided
     /// at construction time.
     #[inline]
-    pub fn buffer(&self, idx: usize) -> ArrayData {
-        self.as_ref()
-            .child(
-                idx + 1,
-                &DType::BYTES,
-                self.metadata().buffer_lens[idx] as usize,
-            )
-            .vortex_expect("VarBinViewArray: buffer child")
-    }
-
-    /// Retrieve an iterator over the raw data buffers.
-    /// These are the BYTE buffers that make up the array's contents.
-    ///
-    /// Example
-    ///
-    /// ```
-    /// use vortex_array::array::VarBinViewArray;
-    /// let array = VarBinViewArray::from_iter_str(["a", "b", "c"]);
-    /// array.buffers().for_each(|block| {
-    ///     // Do something with the `block`
-    /// });
-    /// ```
-    pub fn buffers(&self) -> Buffers {
-        Buffers {
-            index: 0,
-            n_buffers: u32::try_from(self.buffer_count())
-                .unwrap_or_else(|e| vortex_panic!("n_buffers exceeds u32::MAX: {e}")),
-            array: self,
+    pub fn buffer(&self, idx: usize) -> ByteBuffer {
+        if idx >= self.nbuffers() {
+            vortex_panic!(
+                "{idx} buffer index out of bounds, there are {} buffers",
+                self.nbuffers()
+            );
         }
+
+        self.0
+            .byte_buffer(idx + 1)
+            .vortex_expect("Out of bounds view buffer")
+            .clone()
     }
 
+    /// Iterate over the underlying raw data buffers, not including the views buffer.
+    #[inline]
+    pub fn buffers(&self) -> impl Iterator<Item = ByteBuffer> + '_ {
+        self.0.byte_buffers().skip(1)
+    }
+
+    /// Validity of the array
     pub fn validity(&self) -> Validity {
         self.metadata().validity.to_validity(|| {
-            self.as_ref()
-                .child(
-                    (self.metadata().buffer_lens.len() + 1) as _,
-                    &Validity::DTYPE,
-                    self.len(),
-                )
+            self.0
+                .child(0, &Validity::DTYPE, self.len())
                 .vortex_expect("VarBinViewArray: validity child")
         })
     }
@@ -400,22 +340,16 @@ impl VarBinViewArray {
                         }
                     },
                 );
-                VarBinViewArray::try_from(ArrayData::from_arrow(
-                    &string_view_array,
-                    nullability.into(),
-                ))
-                .vortex_expect("StringViewArray to VarBinViewArray downcast")
+                VarBinViewArray::try_from(Array::from_arrow(&string_view_array, nullability.into()))
+                    .vortex_expect("StringViewArray to VarBinViewArray downcast")
             }
             DType::Binary(nullability) => {
                 let binary_view_array = generic_byte_view_builder::<BinaryViewType, _, _>(
                     iter.into_iter(),
                     GenericByteViewBuilder::append_option,
                 );
-                VarBinViewArray::try_from(ArrayData::from_arrow(
-                    &binary_view_array,
-                    nullability.into(),
-                ))
-                .vortex_expect("BinaryViewArray to VarBinViewArray downcast")
+                VarBinViewArray::try_from(Array::from_arrow(&binary_view_array, nullability.into()))
+                    .vortex_expect("BinaryViewArray to VarBinViewArray downcast")
             }
             other => vortex_panic!("VarBinViewArray must be Utf8 or Binary, was {other}"),
         }
@@ -427,7 +361,7 @@ impl VarBinViewArray {
         for s in iter {
             builder.append_value(s);
         }
-        let array = ArrayData::from_arrow(&builder.finish(), false);
+        let array = Array::from_arrow(&builder.finish(), false);
         VarBinViewArray::try_from(array).vortex_expect("VarBinViewArray from StringViewBuilder")
     }
 
@@ -438,7 +372,7 @@ impl VarBinViewArray {
         let mut builder = StringViewBuilder::with_capacity(iter.size_hint().0);
         builder.extend(iter);
 
-        let array = ArrayData::from_arrow(&builder.finish(), true);
+        let array = Array::from_arrow(&builder.finish(), true);
         VarBinViewArray::try_from(array).vortex_expect("VarBinViewArray from StringViewBuilder")
     }
 
@@ -448,7 +382,7 @@ impl VarBinViewArray {
         for b in iter {
             builder.append_value(b);
         }
-        let array = ArrayData::from_arrow(&builder.finish(), false);
+        let array = Array::from_arrow(&builder.finish(), false);
         VarBinViewArray::try_from(array).vortex_expect("VarBinViewArray from StringViewBuilder")
     }
 
@@ -458,53 +392,9 @@ impl VarBinViewArray {
         let iter = iter.into_iter();
         let mut builder = BinaryViewBuilder::with_capacity(iter.size_hint().0);
         builder.extend(iter);
-        let array = ArrayData::from_arrow(&builder.finish(), true);
+        let array = Array::from_arrow(&builder.finish(), true);
         VarBinViewArray::try_from(array).vortex_expect("VarBinViewArray from StringViewBuilder")
     }
-
-    // TODO(aduffy): do we really need to do this with copying?
-    pub fn bytes_at(&self, index: usize) -> VortexResult<Vec<u8>> {
-        let view = self.view_at(index)?;
-        // Expect this to be the common case: strings > 12 bytes.
-        if !view.is_inlined() {
-            let view_ref = view.as_view();
-            let data_buf = slice(
-                self.buffer(view_ref.buffer_index() as usize),
-                view_ref.offset() as usize,
-                (view.len() + view_ref.offset()) as usize,
-            )?
-            .into_primitive()?;
-            Ok(data_buf.maybe_null_slice::<u8>().to_vec())
-        } else {
-            // Return access to the range of bytes around it.
-            Ok(view.as_inlined().value().to_vec())
-        }
-    }
-}
-
-pub struct Views {
-    inner: Buffer,
-    len: usize,
-    idx: usize,
-}
-
-impl Iterator for Views {
-    type Item = BinaryView;
-
-    fn next(&mut self) -> Option<Self::Item> {
-        if self.idx >= self.len {
-            None
-        } else {
-            // SAFETY: constructing the BinaryView against u128 value
-            let mut le_bytes = [0u8; 16];
-            let start = self.idx * VIEW_SIZE_BYTES;
-            let stop = start + VIEW_SIZE_BYTES;
-            le_bytes.copy_from_slice(&self.inner[start..stop]);
-
-            self.idx += 1;
-            Some(BinaryView { le_bytes })
-        }
-    }
 }
 
 // Generic helper to create an Arrow ByteViewBuilder of the appropriate type.
@@ -526,13 +416,13 @@ where
     builder.finish()
 }
 
-impl ArrayTrait for VarBinViewArray {}
+impl ValidateVTable<VarBinViewArray> for VarBinViewEncoding {}
 
-impl IntoCanonical for VarBinViewArray {
-    fn into_canonical(self) -> VortexResult<Canonical> {
-        let nullable = self.dtype().is_nullable();
-        let arrow_self = varbinview_as_arrow(&self);
-        let vortex_array = ArrayData::from_arrow(arrow_self, nullable);
+impl CanonicalVTable<VarBinViewArray> for VarBinViewEncoding {
+    fn into_canonical(&self, array: VarBinViewArray) -> VortexResult<Canonical> {
+        let nullable = array.dtype().is_nullable();
+        let arrow_array = varbinview_as_arrow(&array);
+        let vortex_array = Array::from_arrow(arrow_array, nullable);
 
         Ok(Canonical::VarBinView(VarBinViewArray::try_from(
             vortex_array,
@@ -541,44 +431,34 @@ impl IntoCanonical for VarBinViewArray {
 }
 
 pub(crate) fn varbinview_as_arrow(var_bin_view: &VarBinViewArray) -> ArrayRef {
-    // Views should be buffer of u8
-    let views = var_bin_view
-        .views()
-        .into_primitive()
-        .vortex_expect("VarBinViewArray: views child must be primitive");
-    assert_eq!(views.ptype(), PType::U8);
+    let views = var_bin_view.views();
 
     let nulls = var_bin_view
         .logical_validity()
-        .to_null_buffer()
-        .vortex_expect("VarBinViewArray: validity child must be bool");
+        .vortex_expect("VarBinViewArray: failed to get logical validity")
+        .to_null_buffer();
 
-    let data = (0..var_bin_view.buffer_count())
-        .map(|i| var_bin_view.buffer(i).into_primitive())
-        .collect::<VortexResult<Vec<_>>>()
-        .vortex_expect("VarBinViewArray: bytes children must be primitive");
-    if !data.is_empty() {
-        assert_eq!(data[0].ptype(), PType::U8);
-        assert!(data.iter().map(|d| d.ptype()).all_equal());
-    }
+    let data = (0..var_bin_view.nbuffers())
+        .map(|i| var_bin_view.buffer(i))
+        .collect::<Vec<_>>();
 
     let data = data
         .iter()
-        .map(|p| p.buffer().clone().into_arrow())
+        .map(|p| p.clone().into_arrow_buffer())
         .collect::<Vec<_>>();
 
     // Switch on Arrow DType.
     match var_bin_view.dtype() {
         DType::Binary(_) => Arc::new(unsafe {
             BinaryViewArray::new_unchecked(
-                ScalarBuffer::<u128>::from(views.buffer().clone().into_arrow()),
+                ScalarBuffer::<u128>::from(views.into_byte_buffer().into_arrow_buffer()),
                 data,
                 nulls,
             )
         }),
         DType::Utf8(_) => Arc::new(unsafe {
             StringViewArray::new_unchecked(
-                ScalarBuffer::<u128>::from(views.buffer().clone().into_arrow()),
+                ScalarBuffer::<u128>::from(views.into_byte_buffer().into_arrow_buffer()),
                 data,
                 nulls,
             )
@@ -588,21 +468,21 @@ pub(crate) fn varbinview_as_arrow(var_bin_view: &VarBinViewArray) -> ArrayRef {
 }
 
 impl ValidityVTable<VarBinViewArray> for VarBinViewEncoding {
-    fn is_valid(&self, array: &VarBinViewArray, index: usize) -> bool {
+    fn is_valid(&self, array: &VarBinViewArray, index: usize) -> VortexResult<bool> {
         array.validity().is_valid(index)
     }
 
-    fn logical_validity(&self, array: &VarBinViewArray) -> LogicalValidity {
+    fn logical_validity(&self, array: &VarBinViewArray) -> VortexResult<Mask> {
         array.validity().to_logical(array.len())
     }
 }
 
 impl VisitorVTable<VarBinViewArray> for VarBinViewEncoding {
     fn accept(&self, array: &VarBinViewArray, visitor: &mut dyn ArrayVisitor) -> VortexResult<()> {
-        visitor.visit_child("views", &array.views())?;
-        for i in 0..array.metadata().buffer_lens.len() {
-            visitor.visit_child(format!("bytes_{i}").as_str(), &array.buffer(i))?;
+        for buffer in array.as_ref().byte_buffers() {
+            visitor.visit_buffer(&buffer)?;
         }
+
         visitor.visit_validity(&array.validity())
     }
 }
@@ -635,10 +515,10 @@ impl<'a> FromIterator<Option<&'a str>> for VarBinViewArray {
 mod test {
     use vortex_scalar::Scalar;
 
-    use crate::array::varbinview::{BinaryView, VarBinViewArray, VIEW_SIZE_BYTES};
-    use crate::compute::slice;
-    use crate::compute::unary::scalar_at;
-    use crate::{ArrayLen, Canonical, IntoArrayData, IntoCanonical};
+    use crate::array::varbinview::{BinaryView, VarBinViewArray};
+    use crate::canonical::IntoCanonical;
+    use crate::compute::{scalar_at, slice};
+    use crate::{Canonical, IntoArray};
 
     #[test]
     pub fn varbin_view() {
@@ -684,8 +564,7 @@ mod test {
 
     #[test]
     pub fn binary_view_size_and_alignment() {
-        assert_eq!(size_of::<BinaryView>(), VIEW_SIZE_BYTES);
         assert_eq!(size_of::<BinaryView>(), 16);
-        assert_eq!(align_of::<BinaryView>(), 8);
+        assert_eq!(align_of::<BinaryView>(), 16);
     }
 }
