@@ -2,11 +2,13 @@ use std::sync::Arc;
 
 use arrow_array::cast::AsArray;
 use arrow_array::ArrayRef;
-use vortex_error::{vortex_bail, vortex_err, VortexError, VortexResult};
+use arrow_schema::DataType;
+use vortex_dtype::DType;
+use vortex_error::{vortex_bail, VortexError, VortexResult};
 
-use crate::arrow::FromArrowArray;
+use crate::arrow::{FromArrowArray, IntoArrowArray};
 use crate::encoding::Encoding;
-use crate::{ArrayDType, ArrayData, Canonical, IntoArrayVariant};
+use crate::Array;
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum BinaryOperator {
@@ -19,32 +21,27 @@ pub enum BinaryOperator {
     // Xor,
 }
 
-pub trait BinaryBooleanFn<Array> {
+pub trait BinaryBooleanFn<A> {
     fn binary_boolean(
         &self,
-        array: &Array,
-        other: &ArrayData,
+        array: &A,
+        other: &Array,
         op: BinaryOperator,
-    ) -> VortexResult<ArrayData>;
+    ) -> VortexResult<Option<Array>>;
 }
 
-impl<E: Encoding> BinaryBooleanFn<ArrayData> for E
+impl<E: Encoding> BinaryBooleanFn<Array> for E
 where
     E: BinaryBooleanFn<E::Array>,
-    for<'a> &'a E::Array: TryFrom<&'a ArrayData, Error = VortexError>,
+    for<'a> &'a E::Array: TryFrom<&'a Array, Error = VortexError>,
 {
     fn binary_boolean(
         &self,
-        lhs: &ArrayData,
-        rhs: &ArrayData,
+        lhs: &Array,
+        rhs: &Array,
         op: BinaryOperator,
-    ) -> VortexResult<ArrayData> {
-        let array_ref = <&E::Array>::try_from(lhs)?;
-        let encoding = lhs
-            .encoding()
-            .as_any()
-            .downcast_ref::<E>()
-            .ok_or_else(|| vortex_err!("Mismatched encoding"))?;
+    ) -> VortexResult<Option<Array>> {
+        let (array_ref, encoding) = lhs.try_downcast_ref::<E>()?;
         BinaryBooleanFn::binary_boolean(encoding, array_ref, rhs, op)
     }
 }
@@ -52,34 +49,28 @@ where
 /// Point-wise logical _and_ between two Boolean arrays.
 ///
 /// This method uses Arrow-style null propagation rather than the Kleene logic semantics.
-pub fn and(lhs: impl AsRef<ArrayData>, rhs: impl AsRef<ArrayData>) -> VortexResult<ArrayData> {
+pub fn and(lhs: impl AsRef<Array>, rhs: impl AsRef<Array>) -> VortexResult<Array> {
     binary_boolean(lhs.as_ref(), rhs.as_ref(), BinaryOperator::And)
 }
 
 /// Point-wise Kleene logical _and_ between two Boolean arrays.
-pub fn and_kleene(
-    lhs: impl AsRef<ArrayData>,
-    rhs: impl AsRef<ArrayData>,
-) -> VortexResult<ArrayData> {
+pub fn and_kleene(lhs: impl AsRef<Array>, rhs: impl AsRef<Array>) -> VortexResult<Array> {
     binary_boolean(lhs.as_ref(), rhs.as_ref(), BinaryOperator::AndKleene)
 }
 
 /// Point-wise logical _or_ between two Boolean arrays.
 ///
 /// This method uses Arrow-style null propagation rather than the Kleene logic semantics.
-pub fn or(lhs: impl AsRef<ArrayData>, rhs: impl AsRef<ArrayData>) -> VortexResult<ArrayData> {
+pub fn or(lhs: impl AsRef<Array>, rhs: impl AsRef<Array>) -> VortexResult<Array> {
     binary_boolean(lhs.as_ref(), rhs.as_ref(), BinaryOperator::Or)
 }
 
 /// Point-wise Kleene logical _or_ between two Boolean arrays.
-pub fn or_kleene(
-    lhs: impl AsRef<ArrayData>,
-    rhs: impl AsRef<ArrayData>,
-) -> VortexResult<ArrayData> {
+pub fn or_kleene(lhs: impl AsRef<Array>, rhs: impl AsRef<Array>) -> VortexResult<Array> {
     binary_boolean(lhs.as_ref(), rhs.as_ref(), BinaryOperator::OrKleene)
 }
 
-fn binary_boolean(lhs: &ArrayData, rhs: &ArrayData, op: BinaryOperator) -> VortexResult<ArrayData> {
+pub fn binary_boolean(lhs: &Array, rhs: &Array, op: BinaryOperator) -> VortexResult<Array> {
     if lhs.len() != rhs.len() {
         vortex_bail!("Boolean operations aren't supported on arrays of different lengths")
     }
@@ -92,28 +83,61 @@ fn binary_boolean(lhs: &ArrayData, rhs: &ArrayData, op: BinaryOperator) -> Vorte
         return binary_boolean(rhs, lhs, op);
     }
 
+    // If the RHS is constant and the LHS is Arrow, we can't do any better than arrow_compare.
+    if lhs.is_arrow() && (rhs.is_arrow() || rhs.is_constant()) {
+        return arrow_boolean(lhs.clone(), rhs.clone(), op);
+    }
+
     // Check if either LHS or RHS supports the operation directly.
-    if let Some(f) = lhs.encoding().binary_boolean_fn(lhs, rhs) {
-        return f.binary_boolean(lhs, rhs, op);
-    } else {
-        log::debug!(
-            "No boolean implementation found for LHS {}, RHS {}, and operator {:?}",
-            lhs.encoding().id(),
-            rhs.encoding().id(),
-            op,
+    if let Some(result) = lhs
+        .vtable()
+        .binary_boolean_fn()
+        .and_then(|f| f.binary_boolean(lhs, rhs, op).transpose())
+        .transpose()?
+    {
+        debug_assert_eq!(
+            result.len(),
+            lhs.len(),
+            "Boolean operation length mismatch {}",
+            lhs.encoding()
         );
-    }
-    if let Some(f) = rhs.encoding().binary_boolean_fn(rhs, lhs) {
-        return f.binary_boolean(rhs, lhs, op);
-    } else {
-        log::debug!(
-            "No boolean implementation found for LHS {}, RHS {}, and operator {:?}",
-            rhs.encoding().id(),
-            lhs.encoding().id(),
-            op,
+        debug_assert_eq!(
+            result.dtype(),
+            &DType::Bool((lhs.dtype().is_nullable() || rhs.dtype().is_nullable()).into()),
+            "Boolean operation dtype mismatch {}",
+            lhs.encoding()
         );
+        return Ok(result);
     }
 
+    if let Some(result) = rhs
+        .vtable()
+        .binary_boolean_fn()
+        .and_then(|f| f.binary_boolean(rhs, lhs, op).transpose())
+        .transpose()?
+    {
+        debug_assert_eq!(
+            result.len(),
+            lhs.len(),
+            "Boolean operation length mismatch {}",
+            rhs.encoding()
+        );
+        debug_assert_eq!(
+            result.dtype(),
+            &DType::Bool((lhs.dtype().is_nullable() || rhs.dtype().is_nullable()).into()),
+            "Boolean operation dtype mismatch {}",
+            rhs.encoding()
+        );
+        return Ok(result);
+    }
+
+    log::debug!(
+        "No boolean implementation found for LHS {}, RHS {}, and operator {:?} (or inverse)",
+        rhs.encoding(),
+        lhs.encoding(),
+        op,
+    );
+
     // If neither side implements the trait, then we delegate to Arrow compute.
     arrow_boolean(lhs.clone(), rhs.clone(), op)
 }
@@ -123,20 +147,14 @@ fn binary_boolean(lhs: &ArrayData, rhs: &ArrayData, op: BinaryOperator) -> Vorte
 /// Note that other encodings should handle a constant RHS value, so we can assume here that
 /// the RHS is not constant and expand to a full array.
 pub(crate) fn arrow_boolean(
-    lhs: ArrayData,
-    rhs: ArrayData,
+    lhs: Array,
+    rhs: Array,
     operator: BinaryOperator,
-) -> VortexResult<ArrayData> {
+) -> VortexResult<Array> {
     let nullable = lhs.dtype().is_nullable() || rhs.dtype().is_nullable();
 
-    let lhs = Canonical::Bool(lhs.into_bool()?)
-        .into_arrow()?
-        .as_boolean()
-        .clone();
-    let rhs = Canonical::Bool(rhs.into_bool()?)
-        .into_arrow()?
-        .as_boolean()
-        .clone();
+    let lhs = lhs.into_arrow(&DataType::Boolean)?.as_boolean().clone();
+    let rhs = rhs.into_arrow(&DataType::Boolean)?.as_boolean().clone();
 
     let array = match operator {
         BinaryOperator::And => arrow_arith::boolean::and(&lhs, &rhs)?,
@@ -145,7 +163,7 @@ pub(crate) fn arrow_boolean(
         BinaryOperator::OrKleene => arrow_arith::boolean::or_kleene(&lhs, &rhs)?,
     };
 
-    Ok(ArrayData::from_arrow(Arc::new(array) as ArrayRef, nullable))
+    Ok(Array::from_arrow(Arc::new(array) as ArrayRef, nullable))
 }
 
 #[cfg(test)]
@@ -154,8 +172,9 @@ mod tests {
 
     use super::*;
     use crate::array::BoolArray;
-    use crate::compute::unary::scalar_at;
-    use crate::IntoArrayData;
+    use crate::canonical::IntoArrayVariant;
+    use crate::compute::scalar_at;
+    use crate::IntoArray;
 
     #[rstest]
     #[case(BoolArray::from_iter([Some(true), Some(true), Some(false), Some(false)].into_iter())
@@ -163,7 +182,7 @@ mod tests {
     .into_array())]
     #[case(BoolArray::from_iter([Some(true), Some(false), Some(true), Some(false)].into_iter()).into_array(),
         BoolArray::from_iter([Some(true), Some(true), Some(false), Some(false)].into_iter()).into_array())]
-    fn test_or(#[case] lhs: ArrayData, #[case] rhs: ArrayData) {
+    fn test_or(#[case] lhs: Array, #[case] rhs: Array) {
         let r = or(&lhs, &rhs).unwrap();
 
         let r = r.into_bool().unwrap().into_array();
@@ -185,7 +204,7 @@ mod tests {
     .into_array())]
     #[case(BoolArray::from_iter([Some(true), Some(false), Some(true), Some(false)].into_iter()).into_array(),
         BoolArray::from_iter([Some(true), Some(true), Some(false), Some(false)].into_iter()).into_array())]
-    fn test_and(#[case] lhs: ArrayData, #[case] rhs: ArrayData) {
+    fn test_and(#[case] lhs: Array, #[case] rhs: Array) {
         let r = and(&lhs, &rhs).unwrap().into_bool().unwrap().into_array();
 
         let v0 = scalar_at(&r, 0).unwrap().as_bool().value();
