@@ -4,12 +4,12 @@ use std::fmt::{Debug, Display, Formatter};
 use std::hint;
 
 use itertools::Itertools;
-use vortex_error::{vortex_bail, vortex_err, VortexError, VortexResult};
+use vortex_error::{vortex_bail, VortexError, VortexResult};
 use vortex_scalar::Scalar;
 
-use crate::compute::unary::scalar_at;
+use crate::compute::scalar_at;
 use crate::encoding::Encoding;
-use crate::{ArrayDType, ArrayData};
+use crate::Array;
 
 #[derive(Debug, Copy, Clone)]
 pub enum SearchSortedSide {
@@ -84,6 +84,18 @@ impl SearchResult {
             idx
         }
     }
+
+    /// Apply a transformation to the Found or NotFound index.
+    #[inline]
+    pub fn map<F>(self, f: F) -> SearchResult
+    where
+        F: FnOnce(usize) -> usize,
+    {
+        match self {
+            SearchResult::Found(i) => SearchResult::Found(f(i)),
+            SearchResult::NotFound(i) => SearchResult::NotFound(f(i)),
+        }
+    }
 }
 
 impl Display for SearchResult {
@@ -98,28 +110,18 @@ impl Display for SearchResult {
 /// Searches for value assuming the array is sorted.
 ///
 /// For nullable arrays we assume that the nulls are sorted last, i.e. they're the greatest value
-pub trait SearchSortedFn<Array> {
+pub trait SearchSortedFn<A> {
     fn search_sorted(
         &self,
-        array: &Array,
+        array: &A,
         value: &Scalar,
         side: SearchSortedSide,
     ) -> VortexResult<SearchResult>;
 
-    fn search_sorted_usize(
-        &self,
-        array: &Array,
-        value: usize,
-        side: SearchSortedSide,
-    ) -> VortexResult<SearchResult> {
-        let usize_scalar = Scalar::from(value);
-        self.search_sorted(array, &usize_scalar, side)
-    }
-
     /// Bulk search for many values.
     fn search_sorted_many(
         &self,
-        array: &Array,
+        array: &A,
         values: &[Scalar],
         side: SearchSortedSide,
     ) -> VortexResult<Vec<SearchResult>> {
@@ -128,10 +130,19 @@ pub trait SearchSortedFn<Array> {
             .map(|value| self.search_sorted(array, value, side))
             .try_collect()
     }
+}
+
+pub trait SearchSortedUsizeFn<A> {
+    fn search_sorted_usize(
+        &self,
+        array: &A,
+        value: usize,
+        side: SearchSortedSide,
+    ) -> VortexResult<SearchResult>;
 
     fn search_sorted_usize_many(
         &self,
-        array: &Array,
+        array: &A,
         values: &[usize],
         side: SearchSortedSide,
     ) -> VortexResult<Vec<SearchResult>> {
@@ -142,131 +153,149 @@ pub trait SearchSortedFn<Array> {
     }
 }
 
-impl<E: Encoding> SearchSortedFn<ArrayData> for E
+impl<E: Encoding> SearchSortedFn<Array> for E
 where
     E: SearchSortedFn<E::Array>,
-    for<'a> &'a E::Array: TryFrom<&'a ArrayData, Error = VortexError>,
+    for<'a> &'a E::Array: TryFrom<&'a Array, Error = VortexError>,
 {
     fn search_sorted(
         &self,
-        array: &ArrayData,
+        array: &Array,
         value: &Scalar,
         side: SearchSortedSide,
     ) -> VortexResult<SearchResult> {
-        let array_ref = <&E::Array>::try_from(array)?;
-        let encoding = array
-            .encoding()
-            .as_any()
-            .downcast_ref::<E>()
-            .ok_or_else(|| vortex_err!("Mismatched encoding"))?;
+        let (array_ref, encoding) = array.try_downcast_ref::<E>()?;
         SearchSortedFn::search_sorted(encoding, array_ref, value, side)
     }
 
-    fn search_sorted_usize(
-        &self,
-        array: &ArrayData,
-        value: usize,
-        side: SearchSortedSide,
-    ) -> VortexResult<SearchResult> {
-        let array_ref = <&E::Array>::try_from(array)?;
-        let encoding = array
-            .encoding()
-            .as_any()
-            .downcast_ref::<E>()
-            .ok_or_else(|| vortex_err!("Mismatched encoding"))?;
-        SearchSortedFn::search_sorted_usize(encoding, array_ref, value, side)
-    }
-
     fn search_sorted_many(
         &self,
-        array: &ArrayData,
+        array: &Array,
         values: &[Scalar],
         side: SearchSortedSide,
     ) -> VortexResult<Vec<SearchResult>> {
-        let array_ref = <&E::Array>::try_from(array)?;
-        let encoding = array
-            .encoding()
-            .as_any()
-            .downcast_ref::<E>()
-            .ok_or_else(|| vortex_err!("Mismatched encoding"))?;
+        let (array_ref, encoding) = array.try_downcast_ref::<E>()?;
         SearchSortedFn::search_sorted_many(encoding, array_ref, values, side)
     }
+}
+
+impl<E: Encoding> SearchSortedUsizeFn<Array> for E
+where
+    E: SearchSortedUsizeFn<E::Array>,
+    for<'a> &'a E::Array: TryFrom<&'a Array, Error = VortexError>,
+{
+    fn search_sorted_usize(
+        &self,
+        array: &Array,
+        value: usize,
+        side: SearchSortedSide,
+    ) -> VortexResult<SearchResult> {
+        let (array_ref, encoding) = array.try_downcast_ref::<E>()?;
+        SearchSortedUsizeFn::search_sorted_usize(encoding, array_ref, value, side)
+    }
 
     fn search_sorted_usize_many(
         &self,
-        array: &ArrayData,
+        array: &Array,
         values: &[usize],
         side: SearchSortedSide,
     ) -> VortexResult<Vec<SearchResult>> {
-        let array_ref = <&E::Array>::try_from(array)?;
-        let encoding = array
-            .encoding()
-            .as_any()
-            .downcast_ref::<E>()
-            .ok_or_else(|| vortex_err!("Mismatched encoding"))?;
-        SearchSortedFn::search_sorted_usize_many(encoding, array_ref, values, side)
+        let (array_ref, encoding) = array.try_downcast_ref::<E>()?;
+        SearchSortedUsizeFn::search_sorted_usize_many(encoding, array_ref, values, side)
     }
 }
 
 pub fn search_sorted<T: Into<Scalar>>(
-    array: &ArrayData,
+    array: &Array,
     target: T,
     side: SearchSortedSide,
 ) -> VortexResult<SearchResult> {
-    let scalar = target.into().cast(array.dtype())?;
+    let Ok(scalar) = target.into().cast(array.dtype()) else {
+        // Try to downcast the usize ot the array type, if the downcast fails, then we know the
+        // usize is too large and the value is greater than the highest value in the array.
+        return Ok(SearchResult::NotFound(array.len()));
+    };
+
     if scalar.is_null() {
         vortex_bail!("Search sorted with null value is not supported");
     }
 
-    if let Some(f) = array.encoding().search_sorted_fn() {
+    if let Some(f) = array.vtable().search_sorted_fn() {
         return f.search_sorted(array, &scalar, side);
     }
 
     // Fallback to a generic search_sorted using scalar_at
-    if array.encoding().scalar_at_fn().is_some() {
+    if array.vtable().scalar_at_fn().is_some() {
         return Ok(SearchSorted::search_sorted(array, &scalar, side));
     }
 
     vortex_bail!(
         NotImplemented: "search_sorted",
-        array.encoding().id()
+        array.encoding()
     )
 }
 
 pub fn search_sorted_usize(
-    array: &ArrayData,
+    array: &Array,
     target: usize,
     side: SearchSortedSide,
 ) -> VortexResult<SearchResult> {
-    if let Some(f) = array.encoding().search_sorted_fn() {
+    if let Some(f) = array.vtable().search_sorted_usize_fn() {
         return f.search_sorted_usize(array, target, side);
     }
 
-    // Fallback to a generic search_sorted using scalar_at
-    if array.encoding().scalar_at_fn().is_some() {
-        let scalar = Scalar::primitive(target as u64, array.dtype().nullability());
-        return Ok(SearchSorted::search_sorted(array, &scalar, side));
+    // Otherwise, convert the target into a scalar to try the search_sorted_fn
+    let Ok(target) = Scalar::from(target).cast(array.dtype()) else {
+        return Ok(SearchResult::NotFound(array.len()));
+    };
+
+    // Try the non-usize search sorted
+    if let Some(f) = array.vtable().search_sorted_fn() {
+        return f.search_sorted(array, &target, side);
+    }
+
+    // Or fallback all the way to a generic search_sorted using scalar_at
+    if array.vtable().scalar_at_fn().is_some() {
+        // Try to downcast the usize to the array type, if the downcast fails, then we know the
+        // usize is too large and the value is greater than the highest value in the array.
+        let Ok(target) = target.cast(array.dtype()) else {
+            return Ok(SearchResult::NotFound(array.len()));
+        };
+        return Ok(SearchSorted::search_sorted(array, &target, side));
     }
 
     vortex_bail!(
     NotImplemented: "search_sorted_usize",
-        array.encoding().id()
+        array.encoding()
     )
 }
 
 /// Search for many elements in the array.
 pub fn search_sorted_many<T: Into<Scalar> + Clone>(
-    array: &ArrayData,
+    array: &Array,
     targets: &[T],
     side: SearchSortedSide,
 ) -> VortexResult<Vec<SearchResult>> {
-    if let Some(f) = array.encoding().search_sorted_fn() {
-        let values: Vec<Scalar> = targets
+    if let Some(f) = array.vtable().search_sorted_fn() {
+        let mut too_big_cast_idxs = Vec::new();
+        let values = targets
             .iter()
-            .map(|t| t.clone().into().cast(array.dtype()))
-            .try_collect()?;
+            .cloned()
+            .enumerate()
+            .filter_map(|(i, t)| {
+                let Ok(c) = t.into().cast(array.dtype()) else {
+                    too_big_cast_idxs.push(i);
+                    return None;
+                };
+                Some(c)
+            })
+            .collect::<Vec<_>>();
 
-        return f.search_sorted_many(array, &values, side);
+        let mut results = f.search_sorted_many(array, &values, side)?;
+        for too_big_idx in too_big_cast_idxs {
+            results.insert(too_big_idx, SearchResult::NotFound(array.len()));
+        }
+        return Ok(results);
     }
 
     // Call in loop and collect
@@ -278,11 +307,11 @@ pub fn search_sorted_many<T: Into<Scalar> + Clone>(
 
 // Native functions for each of the values, cast up to u64 or down to something lower.
 pub fn search_sorted_usize_many(
-    array: &ArrayData,
+    array: &Array,
     targets: &[usize],
     side: SearchSortedSide,
 ) -> VortexResult<Vec<SearchResult>> {
-    if let Some(f) = array.encoding().search_sorted_fn() {
+    if let Some(f) = array.vtable().search_sorted_usize_fn() {
         return f.search_sorted_usize_many(array, targets, side);
     }
 
@@ -294,6 +323,8 @@ pub fn search_sorted_usize_many(
 }
 
 pub trait IndexOrd<V> {
+    /// PartialOrd of the value at index `idx` with `elem`.
+    /// For example, if self\[idx\] > elem, return Some(Greater).
     fn index_cmp(&self, idx: usize, elem: &V) -> Option<Ordering>;
 
     fn index_lt(&self, idx: usize, elem: &V) -> bool {
@@ -444,7 +475,7 @@ fn search_sorted_side_idx<F: FnMut(usize) -> Ordering>(
     }
 }
 
-impl IndexOrd<Scalar> for ArrayData {
+impl IndexOrd<Scalar> for Array {
     fn index_cmp(&self, idx: usize, elem: &Scalar) -> Option<Ordering> {
         let scalar_a = scalar_at(self, idx).ok()?;
         scalar_a.partial_cmp(elem)
@@ -458,7 +489,7 @@ impl<T: PartialOrd> IndexOrd<T> for [T] {
     }
 }
 
-impl Len for ArrayData {
+impl Len for Array {
     #[allow(clippy::same_name_method)]
     fn len(&self) -> usize {
         Self::len(self)
@@ -473,7 +504,11 @@ impl<T> Len for [T] {
 
 #[cfg(test)]
 mod test {
+    use vortex_buffer::buffer;
+
     use crate::compute::search_sorted::{SearchResult, SearchSorted, SearchSortedSide};
+    use crate::compute::{search_sorted, search_sorted_many};
+    use crate::IntoArray;
 
     #[test]
     fn left_side_equal() {
@@ -522,4 +557,18 @@ mod test {
         assert_eq!(arr[res.to_index() - 1], 9);
         assert_eq!(res, SearchResult::Found(13));
     }
+
+    #[test]
+    fn failed_cast() {
+        let arr = buffer![0u8, 1, 2, 3, 4, 5, 6, 7, 8, 9, 9, 9, 9].into_array();
+        let res = search_sorted(&arr, 256, SearchSortedSide::Left).unwrap();
+        assert_eq!(res, SearchResult::NotFound(arr.len()));
+    }
+
+    #[test]
+    fn search_sorted_many_failed_cast() {
+        let arr = buffer![0u8, 1, 2, 3, 4, 5, 6, 7, 8, 9, 9, 9, 9].into_array();
+        let res = search_sorted_many(&arr, &[256], SearchSortedSide::Left).unwrap();
+        assert_eq!(res, vec![SearchResult::NotFound(arr.len())]);
+    }
 }
