@@ -7,23 +7,22 @@ use vortex_dtype::match_each_integer_ptype;
 use vortex_error::{vortex_bail, VortexResult};
 use vortex_scalar::Scalar;
 
-use crate::compute::unary::subtract_scalar;
-use crate::compute::{search_sorted, slice, take, SearchSortedSide, TakeOptions};
-use crate::stats::{ArrayStatistics, Stat};
+use crate::compute::{search_sorted_usize, slice, sub_scalar, take, SearchSortedSide};
+use crate::stats::Stat;
 use crate::stream::ArrayStream;
 use crate::variants::PrimitiveArrayTrait;
-use crate::{ArrayDType, ArrayData, IntoArrayData, IntoArrayVariant};
+use crate::{Array, IntoArray, IntoArrayVariant};
 
 #[pin_project]
 pub struct TakeRows<R: ArrayStream> {
     #[pin]
     reader: R,
-    indices: ArrayData,
+    indices: Array,
     row_offset: usize,
 }
 
 impl<R: ArrayStream> TakeRows<R> {
-    pub fn try_new(reader: R, indices: ArrayData) -> VortexResult<Self> {
+    pub fn try_new(reader: R, indices: Array) -> VortexResult<Self> {
         if !indices.is_empty() {
             if !indices.statistics().compute_is_sorted().unwrap_or(false) {
                 vortex_bail!("Indices must be sorted to take from IPC stream")
@@ -45,7 +44,7 @@ impl<R: ArrayStream> TakeRows<R> {
             if indices.dtype().is_signed_int()
                 && indices
                     .statistics()
-                    .compute_as_cast::<i64>(Stat::Min)
+                    .compute_as::<i64>(Stat::Min)
                     .map(|min| min < 0)
                     .unwrap_or(true)
             {
@@ -62,7 +61,7 @@ impl<R: ArrayStream> TakeRows<R> {
 }
 
 impl<R: ArrayStream> Stream for TakeRows<R> {
-    type Item = VortexResult<ArrayData>;
+    type Item = VortexResult<Array>;
 
     fn poll_next(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
         let mut this = self.project();
@@ -73,8 +72,9 @@ impl<R: ArrayStream> Stream for TakeRows<R> {
 
         while let Some(batch) = ready!(this.reader.as_mut().poll_next(cx)?) {
             let curr_offset = *this.row_offset;
-            let left = search_sorted(this.indices, curr_offset, SearchSortedSide::Left)?.to_index();
-            let right = search_sorted(
+            let left =
+                search_sorted_usize(this.indices, curr_offset, SearchSortedSide::Left)?.to_index();
+            let right = search_sorted_usize(
                 this.indices,
                 curr_offset + batch.len(),
                 SearchSortedSide::Left,
@@ -91,13 +91,9 @@ impl<R: ArrayStream> Stream for TakeRows<R> {
             //  onto a worker pool.
             let indices_for_batch = slice(this.indices, left, right)?.into_primitive()?;
             let shifted_arr = match_each_integer_ptype!(indices_for_batch.ptype(), |$T| {
-                subtract_scalar(&indices_for_batch.into_array(), &Scalar::from(curr_offset as $T))?
+                sub_scalar(&indices_for_batch.into_array(), Scalar::from(curr_offset as $T))?
             });
-            return Poll::Ready(
-                take(&batch, &shifted_arr, TakeOptions::default())
-                    .map(Some)
-                    .transpose(),
-            );
+            return Poll::Ready(take(&batch, &shifted_arr).map(Some).transpose());
         }
 
         Poll::Ready(None)
