@@ -2,23 +2,23 @@ use std::fmt;
 
 use humansize::{format_size, DECIMAL};
 use serde::ser::Error;
-use vortex_buffer::Buffer;
+use vortex_buffer::ByteBuffer;
 use vortex_error::{VortexError, VortexResult};
 
 use crate::array::ChunkedEncoding;
-use crate::encoding::EncodingVTable;
 use crate::visitor::ArrayVisitor;
-use crate::ArrayData;
+use crate::vtable::EncodingVTable;
+use crate::Array;
 
-impl ArrayData {
+impl Array {
     pub fn tree_display(&self) -> TreeDisplayWrapper {
         TreeDisplayWrapper(self)
     }
 }
 
-pub struct TreeDisplayWrapper<'a>(&'a ArrayData);
+pub struct TreeDisplayWrapper<'a>(&'a Array);
 impl<'a> TreeDisplayWrapper<'a> {
-    pub fn new(array: &'a ArrayData) -> Self {
+    pub fn new(array: &'a Array) -> Self {
         Self(array)
     }
 }
@@ -42,47 +42,45 @@ pub struct TreeFormatter<'a, 'b: 'a> {
 /// TODO(ngates): I think we want to go back to the old explicit style. It gives arrays more
 ///  control over how their metadata etc is displayed.
 impl<'a, 'b: 'a> ArrayVisitor for TreeFormatter<'a, 'b> {
-    fn visit_child(&mut self, name: &str, array: &ArrayData) -> VortexResult<()> {
-        array.with_dyn(|a| {
-            let nbytes = a.nbytes();
-            let total_size = self.total_size.unwrap_or(nbytes);
-            writeln!(
-                self.fmt,
-                "{}{}: {} nbytes={} ({:.2}%)",
-                self.indent,
-                name,
-                array,
-                format_size(nbytes, DECIMAL),
-                100f64 * nbytes as f64 / total_size as f64
-            )?;
-            self.indent(|i| writeln!(i.fmt, "{}metadata: {}", i.indent, array.array_metadata()))?;
+    fn visit_child(&mut self, name: &str, array: &Array) -> VortexResult<()> {
+        let nbytes = array.nbytes();
+        let total_size = self.total_size.unwrap_or(nbytes);
+        writeln!(
+            self.fmt,
+            "{}{}: {} nbytes={} ({:.2}%)",
+            self.indent,
+            name,
+            array,
+            format_size(nbytes, DECIMAL),
+            100f64 * nbytes as f64 / total_size as f64
+        )?;
+        self.indent(|i| {
+            write!(i.fmt, "{}metadata: ", i.indent)?;
+            array.vtable().display_metadata(array, i.fmt)?;
+            writeln!(i.fmt)
+        })?;
 
-            let old_total_size = self.total_size;
-            if array.is_encoding(ChunkedEncoding.id()) {
-                // Clear the total size so each chunk is treated as a new root.
-                self.total_size = None
-            } else {
-                self.total_size = Some(total_size);
-            }
+        let old_total_size = self.total_size;
+        if array.is_encoding(ChunkedEncoding.id()) {
+            // Clear the total size so each chunk is treated as a new root.
+            self.total_size = None
+        } else {
+            self.total_size = Some(total_size);
+        }
 
-            self.indent(|i| {
-                array
-                    .encoding()
-                    .accept(array, i)
-                    .map_err(fmt::Error::custom)
-            })
+        self.indent(|i| array.vtable().accept(array, i).map_err(fmt::Error::custom))
             .map_err(VortexError::from)?;
 
-            self.total_size = old_total_size;
-            Ok(())
-        })
+        self.total_size = old_total_size;
+        Ok(())
     }
 
-    fn visit_buffer(&mut self, buffer: &Buffer) -> VortexResult<()> {
+    fn visit_buffer(&mut self, buffer: &ByteBuffer) -> VortexResult<()> {
         Ok(writeln!(
             self.fmt,
-            "{}buffer: {}",
+            "{}buffer (align={}): {}",
             self.indent,
+            buffer.alignment(),
             format_size(buffer.len(), DECIMAL)
         )?)
     }
