@@ -4,60 +4,48 @@ use std::fmt::{Debug, Display};
 use std::ops::BitAnd;
 
 use arrow_buffer::{BooleanBuffer, BooleanBufferBuilder, NullBuffer};
-use num_traits::AsPrimitive;
 use serde::{Deserialize, Serialize};
 use vortex_dtype::{DType, Nullability};
-use vortex_error::{
-    vortex_bail, vortex_err, vortex_panic, VortexError, VortexExpect as _, VortexResult,
-};
+use vortex_error::{vortex_bail, vortex_err, vortex_panic, VortexExpect as _, VortexResult};
+use vortex_mask::{Mask, MaskValues};
 
 use crate::array::{BoolArray, ConstantArray};
-use crate::compute::unary::scalar_at;
-use crate::compute::{filter, slice, take, FilterMask, TakeOptions};
-use crate::encoding::Encoding;
-use crate::stats::ArrayStatistics;
-use crate::{ArrayDType, ArrayData, IntoArrayData, IntoArrayVariant};
+use crate::compute::{filter, scalar_at, slice, take};
+use crate::patches::Patches;
+use crate::{Array, IntoArray, IntoArrayVariant};
 
-pub trait ValidityVTable<Array> {
-    // TODO(ngates): can we implement this based on logical validity? Or is that too expensive?
-    fn is_valid(&self, array: &Array, index: usize) -> bool;
-    fn logical_validity(&self, array: &Array) -> LogicalValidity;
-}
-
-impl<E: Encoding> ValidityVTable<ArrayData> for E
-where
-    E: ValidityVTable<E::Array>,
-    for<'a> &'a E::Array: TryFrom<&'a ArrayData, Error = VortexError>,
-{
-    fn is_valid(&self, array: &ArrayData, index: usize) -> bool {
-        let array_ref =
-            <&E::Array>::try_from(array).vortex_expect("Failed to get array as reference");
-        let encoding = array
-            .encoding()
-            .as_any()
-            .downcast_ref::<E>()
-            .vortex_expect("Failed to downcast encoding");
-        ValidityVTable::is_valid(encoding, array_ref, index)
+impl Array {
+    /// Return whether the element at the given index is valid (true) or null (false).
+    pub fn is_valid(&self, index: usize) -> VortexResult<bool> {
+        self.vtable().is_valid(self, index)
     }
 
-    fn logical_validity(&self, array: &ArrayData) -> LogicalValidity {
-        let array_ref =
-            <&E::Array>::try_from(array).vortex_expect("Failed to get array as reference");
-        let encoding = array
-            .encoding()
-            .as_any()
-            .downcast_ref::<E>()
-            .vortex_expect("Failed to downcast encoding");
-        ValidityVTable::logical_validity(encoding, array_ref)
+    /// Return the number of null elements in the array.
+    pub fn null_count(&self) -> VortexResult<usize> {
+        self.vtable().null_count(self)
+    }
+
+    /// Return the logical validity of the array if nullable, and None if non-nullable.
+    pub fn logical_validity(&self) -> VortexResult<Mask> {
+        self.vtable().logical_validity(self)
     }
 }
 
-pub trait ArrayValidity {
-    fn is_valid(&self, index: usize) -> bool;
-    fn logical_validity(&self) -> LogicalValidity;
-}
-
-#[derive(Clone, Debug, Serialize, Deserialize)]
+#[derive(
+    Copy,
+    Clone,
+    Debug,
+    Serialize,
+    Deserialize,
+    rkyv::Archive,
+    rkyv::Portable,
+    rkyv::Serialize,
+    rkyv::Deserialize,
+    rkyv::bytecheck::CheckBytes,
+)]
+#[rkyv(as = ValidityMetadata)]
+#[bytecheck(crate = rkyv::bytecheck)]
+#[repr(u8)]
 pub enum ValidityMetadata {
     NonNullable,
     AllValid,
@@ -74,7 +62,7 @@ impl Display for ValidityMetadata {
 impl ValidityMetadata {
     pub fn to_validity<F>(&self, array_fn: F) -> Validity
     where
-        F: FnOnce() -> ArrayData,
+        F: FnOnce() -> Array,
     {
         match self {
             Self::NonNullable => Validity::NonNullable,
@@ -95,7 +83,7 @@ pub enum Validity {
     /// All items are null
     AllInvalid,
     /// Specified items are null
-    Array(ArrayData),
+    Array(Array),
 }
 
 impl Validity {
@@ -144,7 +132,7 @@ impl Validity {
     }
 
     /// If Validity is [`Validity::Array`], returns the array, otherwise returns `None`.
-    pub fn into_array(self) -> Option<ArrayData> {
+    pub fn into_array(self) -> Option<Array> {
         match self {
             Self::Array(a) => Some(a),
             _ => None,
@@ -152,7 +140,7 @@ impl Validity {
     }
 
     /// If Validity is [`Validity::Array`], returns a reference to the array array, otherwise returns `None`.
-    pub fn as_array(&self) -> Option<&ArrayData> {
+    pub fn as_array(&self) -> Option<&Array> {
         match self {
             Self::Array(a) => Some(a),
             _ => None,
@@ -168,25 +156,23 @@ impl Validity {
 
     /// Returns whether the `index` item is valid.
     #[inline]
-    pub fn is_valid(&self, index: usize) -> bool {
-        match self {
+    pub fn is_valid(&self, index: usize) -> VortexResult<bool> {
+        Ok(match self {
             Self::NonNullable | Self::AllValid => true,
             Self::AllInvalid => false,
-            Self::Array(a) => scalar_at(a, index)
-                .and_then(|s| bool::try_from(&s))
-                .unwrap_or_else(|err| {
-                    vortex_panic!(
-                        err,
-                        "Failed to get bool from Validity Array at index {}",
-                        index
-                    )
-                }),
-        }
+            Self::Array(a) => {
+                let scalar = scalar_at(a, index)?;
+                scalar
+                    .as_bool()
+                    .value()
+                    .vortex_expect("Validity must be non-nullable")
+            }
+        })
     }
 
     #[inline]
-    pub fn is_null(&self, index: usize) -> bool {
-        !self.is_valid(index)
+    pub fn is_null(&self, index: usize) -> VortexResult<bool> {
+        Ok(!self.is_valid(index)?)
     }
 
     pub fn slice(&self, start: usize, stop: usize) -> VortexResult<Self> {
@@ -196,47 +182,57 @@ impl Validity {
         }
     }
 
-    pub fn take(&self, indices: &ArrayData, options: TakeOptions) -> VortexResult<Self> {
+    pub fn take(&self, indices: &Array) -> VortexResult<Self> {
         match self {
             Self::NonNullable => Ok(Self::NonNullable),
             Self::AllValid => Ok(Self::AllValid),
             Self::AllInvalid => Ok(Self::AllInvalid),
-            Self::Array(a) => Ok(Self::Array(take(a, indices, options)?)),
+            Self::Array(a) => Ok(Self::Array(take(a, indices)?)),
         }
     }
 
-    pub fn filter(&self, mask: &FilterMask) -> VortexResult<Self> {
+    /// Take the validity buffer at the provided indices.
+    ///
+    /// # Safety
+    ///
+    /// It is assumed the caller has checked that all indices are <= the length of this validity
+    /// buffer.
+    ///
+    /// Failure to do so may result in UB.
+    pub unsafe fn take_unchecked(&self, indices: &Array) -> VortexResult<Self> {
+        match self {
+            Self::NonNullable => Ok(Self::NonNullable),
+            Self::AllValid => Ok(Self::AllValid),
+            Self::AllInvalid => Ok(Self::AllInvalid),
+            Self::Array(a) => {
+                let taken = if let Some(take_fn) = a.vtable().take_fn() {
+                    unsafe { take_fn.take_unchecked(a, indices) }
+                } else {
+                    take(a, indices)
+                };
+
+                taken.map(Self::Array)
+            }
+        }
+    }
+
+    pub fn filter(&self, mask: &Mask) -> VortexResult<Self> {
         // NOTE(ngates): we take the mask as a reference to avoid the caller cloning unnecessarily
         //  if we happen to be NonNullable, AllValid, or AllInvalid.
         match self {
             v @ (Validity::NonNullable | Validity::AllValid | Validity::AllInvalid) => {
                 Ok(v.clone())
             }
-            Validity::Array(arr) => Ok(Validity::Array(filter(arr, mask.clone())?)),
+            Validity::Array(arr) => Ok(Validity::Array(filter(arr, mask)?)),
         }
     }
 
-    pub fn to_logical(&self, length: usize) -> LogicalValidity {
-        match self {
-            Self::NonNullable => LogicalValidity::AllValid(length),
-            Self::AllValid => LogicalValidity::AllValid(length),
-            Self::AllInvalid => LogicalValidity::AllInvalid(length),
-            Self::Array(a) => {
-                // Logical validity should map into AllValid/AllInvalid where possible.
-                if a.statistics().compute_min::<bool>().unwrap_or(false) {
-                    LogicalValidity::AllValid(length)
-                } else if a
-                    .statistics()
-                    .compute_max::<bool>()
-                    .map(|m| !m)
-                    .unwrap_or(false)
-                {
-                    LogicalValidity::AllInvalid(length)
-                } else {
-                    LogicalValidity::Array(a.clone())
-                }
-            }
-        }
+    pub fn to_logical(&self, length: usize) -> VortexResult<Mask> {
+        Ok(match self {
+            Self::NonNullable | Self::AllValid => Mask::AllTrue(length),
+            Self::AllInvalid => Mask::AllFalse(length),
+            Self::Array(a) => Mask::try_from(a.clone())?,
+        })
     }
 
     /// Logically & two Validity values of the same length
@@ -270,24 +266,25 @@ impl Validity {
         Ok(validity)
     }
 
-    pub fn patch<P: AsPrimitive<usize>>(
-        self,
-        len: usize,
-        positions: &[P],
-        patches: Validity,
-    ) -> VortexResult<Self> {
-        if let Some(last_pos) = positions.last() {
-            if last_pos.as_() >= len {
-                vortex_bail!(OutOfBounds: last_pos.as_(), 0, len)
+    pub fn patch(self, len: usize, indices: &Array, patches: Validity) -> VortexResult<Self> {
+        match (&self, &patches) {
+            (Validity::NonNullable, Validity::NonNullable) => return Ok(Validity::NonNullable),
+            (Validity::NonNullable, _) => {
+                vortex_bail!("Can't patch a non-nullable validity with nullable validity")
             }
-        }
+            (_, Validity::NonNullable) => {
+                vortex_bail!("Can't patch a nullable validity with non-nullable validity")
+            }
+            (Validity::AllValid, Validity::AllValid) => return Ok(Validity::AllValid),
+            (Validity::AllInvalid, Validity::AllInvalid) => return Ok(Validity::AllInvalid),
+            _ => {}
+        };
 
-        if matches!(self, Validity::NonNullable | Validity::AllValid)
-            && matches!(patches, Validity::NonNullable | Validity::AllValid)
-            || self == patches
-        {
-            return Ok(self);
-        }
+        let own_nullability = if self == Validity::NonNullable {
+            Nullability::NonNullable
+        } else {
+            Nullability::Nullable
+        };
 
         let source = match self {
             Validity::NonNullable => BoolArray::from(BooleanBuffer::new_set(len)),
@@ -297,13 +294,18 @@ impl Validity {
         };
 
         let patch_values = match patches {
-            Validity::NonNullable => BoolArray::from(BooleanBuffer::new_set(positions.len())),
-            Validity::AllValid => BoolArray::from(BooleanBuffer::new_set(positions.len())),
-            Validity::AllInvalid => BoolArray::from(BooleanBuffer::new_unset(positions.len())),
+            Validity::NonNullable => BoolArray::from(BooleanBuffer::new_set(indices.len())),
+            Validity::AllValid => BoolArray::from(BooleanBuffer::new_set(indices.len())),
+            Validity::AllInvalid => BoolArray::from(BooleanBuffer::new_unset(indices.len())),
             Validity::Array(a) => a.into_bool()?,
         };
 
-        Validity::try_from(source.patch(positions, patch_values)?.into_array())
+        let patches = Patches::new(len, indices.clone(), patch_values.into_array());
+
+        Ok(Self::from_array(
+            source.patch(patches)?.into_array(),
+            own_nullability,
+        ))
     }
 
     /// Convert into a nullable variant
@@ -313,6 +315,20 @@ impl Validity {
             _ => self,
         }
     }
+
+    /// Create Validity from boolean array with given nullability of the array.
+    ///
+    /// Note: You want to pass the nullability of parent array and not the nullability of the validity array itself
+    ///     as that is always nonnullable
+    pub fn from_array(value: Array, nullability: Nullability) -> Self {
+        if !matches!(value.dtype(), DType::Bool(Nullability::NonNullable)) {
+            vortex_panic!("Expected a non-nullable boolean array")
+        }
+        match nullability {
+            Nullability::NonNullable => Self::NonNullable,
+            Nullability::Nullable => Self::Array(value),
+        }
+    }
 }
 
 impl PartialEq for Validity {
@@ -357,24 +373,16 @@ impl From<NullBuffer> for Validity {
     }
 }
 
-impl TryFrom<ArrayData> for Validity {
-    type Error = VortexError;
-
-    fn try_from(value: ArrayData) -> Result<Self, Self::Error> {
-        LogicalValidity::try_from(value).map(|a| a.into_validity())
-    }
-}
-
-impl FromIterator<LogicalValidity> for Validity {
-    fn from_iter<T: IntoIterator<Item = LogicalValidity>>(iter: T) -> Self {
-        let validities: Vec<LogicalValidity> = iter.into_iter().collect();
+impl FromIterator<Mask> for Validity {
+    fn from_iter<T: IntoIterator<Item = Mask>>(iter: T) -> Self {
+        let validities: Vec<Mask> = iter.into_iter().collect();
 
         // If they're all valid, then return a single validity.
-        if validities.iter().all(|v| v.all_valid()) {
+        if validities.iter().all(|v| v.all_true()) {
             return Self::AllValid;
         }
         // If they're all invalid, then return a single invalidity.
-        if validities.iter().all(|v| v.all_invalid()) {
+        if validities.iter().all(|v| v.all_false()) {
             return Self::AllInvalid;
         }
 
@@ -382,14 +390,10 @@ impl FromIterator<LogicalValidity> for Validity {
         let mut buffer = BooleanBufferBuilder::new(validities.iter().map(|v| v.len()).sum());
         for validity in validities {
             match validity {
-                LogicalValidity::AllValid(count) => buffer.append_n(count, true),
-                LogicalValidity::AllInvalid(count) => buffer.append_n(count, false),
-                LogicalValidity::Array(array) => {
-                    let array_buffer = array
-                        .into_bool()
-                        .vortex_expect("Failed to get Validity Array as BoolArray")
-                        .boolean_buffer();
-                    buffer.append_buffer(&array_buffer);
+                Mask::AllTrue(count) => buffer.append_n(count, true),
+                Mask::AllFalse(count) => buffer.append_n(count, false),
+                Mask::Values(values) => {
+                    buffer.append_buffer(values.boolean_buffer());
                 }
             };
         }
@@ -400,150 +404,74 @@ impl FromIterator<LogicalValidity> for Validity {
 
 impl FromIterator<bool> for Validity {
     fn from_iter<T: IntoIterator<Item = bool>>(iter: T) -> Self {
-        Self::Array(BoolArray::from_iter(iter).into_array())
+        Validity::from(BooleanBuffer::from_iter(iter))
     }
 }
 
-#[derive(Clone, Debug)]
-pub enum LogicalValidity {
-    AllValid(usize),
-    AllInvalid(usize),
-    Array(ArrayData),
-}
-
-impl LogicalValidity {
-    pub fn try_new_from_array(array: ArrayData) -> VortexResult<Self> {
-        if !matches!(array.dtype(), &Validity::DTYPE) {
-            vortex_bail!("Expected a non-nullable boolean array");
-        }
-
-        let true_count = array.statistics().compute_true_count().ok_or_else(|| {
-            vortex_err!(
-                "Failed to compute true count from validity array {:#?}",
-                array
-            )
-        })?;
-        if true_count == array.len() {
-            return Ok(Self::AllValid(array.len()));
-        } else if true_count == 0 {
-            return Ok(Self::AllInvalid(array.len()));
-        }
-
-        Ok(Self::Array(array))
-    }
-
-    pub fn to_null_buffer(&self) -> VortexResult<Option<NullBuffer>> {
-        match self {
-            Self::AllValid(_) => Ok(None),
-            Self::AllInvalid(l) => Ok(Some(NullBuffer::new_null(*l))),
-            Self::Array(a) => Ok(Some(NullBuffer::new(
-                a.clone().into_bool()?.boolean_buffer(),
-            ))),
-        }
-    }
-
-    pub fn all_valid(&self) -> bool {
-        matches!(self, Self::AllValid(_))
-    }
-
-    pub fn all_invalid(&self) -> bool {
-        matches!(self, Self::AllInvalid(_))
-    }
-
-    pub fn len(&self) -> usize {
-        match self {
-            Self::AllValid(n) => *n,
-            Self::AllInvalid(n) => *n,
-            Self::Array(a) => a.len(),
-        }
-    }
-
-    pub fn is_empty(&self) -> bool {
-        match self {
-            Self::AllValid(n) => *n == 0,
-            Self::AllInvalid(n) => *n == 0,
-            Self::Array(a) => a.is_empty(),
-        }
-    }
-
-    pub fn into_validity(self) -> Validity {
-        match self {
-            Self::AllValid(_) => Validity::AllValid,
-            Self::AllInvalid(_) => Validity::AllInvalid,
-            Self::Array(a) => Validity::Array(a),
-        }
-    }
-
-    pub fn null_count(&self, length: usize) -> VortexResult<usize> {
-        match self {
-            Self::AllValid(_) => Ok(0),
-            Self::AllInvalid(_) => Ok(length),
-            Self::Array(a) => {
-                let validity_len = a.len();
-                if validity_len != length {
-                    vortex_bail!(
-                        "Validity array length {} doesn't match array length {}",
-                        validity_len,
-                        length
-                    )
-                }
-                let true_count = a.statistics().compute_true_count().ok_or_else(|| {
-                    vortex_err!("Failed to compute true count from validity array")
-                })?;
-                Ok(length - true_count)
-            }
+impl From<Nullability> for Validity {
+    fn from(value: Nullability) -> Self {
+        match value {
+            Nullability::NonNullable => Validity::NonNullable,
+            Nullability::Nullable => Validity::AllValid,
         }
     }
 }
 
-impl TryFrom<ArrayData> for LogicalValidity {
-    type Error = VortexError;
-
-    fn try_from(array: ArrayData) -> VortexResult<Self> {
-        Self::try_new_from_array(array)
+impl Validity {
+    pub fn from_mask(mask: Mask, nullability: Nullability) -> Self {
+        assert!(
+            nullability == Nullability::Nullable || matches!(mask, Mask::AllTrue(_)),
+            "NonNullable validity must be AllValid",
+        );
+        match mask {
+            Mask::AllTrue(_) => match nullability {
+                Nullability::NonNullable => Validity::NonNullable,
+                Nullability::Nullable => Validity::AllValid,
+            },
+            Mask::AllFalse(_) => Validity::AllInvalid,
+            Mask::Values(values) => Validity::Array(values.into_array()),
+        }
     }
 }
 
-impl IntoArrayData for LogicalValidity {
-    fn into_array(self) -> ArrayData {
+impl IntoArray for Mask {
+    fn into_array(self) -> Array {
         match self {
-            Self::AllValid(len) => ConstantArray::new(true, len).into_array(),
-            Self::AllInvalid(len) => ConstantArray::new(false, len).into_array(),
-            Self::Array(a) => a,
+            Self::AllTrue(len) => ConstantArray::new(true, len).into_array(),
+            Self::AllFalse(len) => ConstantArray::new(false, len).into_array(),
+            Self::Values(a) => a.into_array(),
         }
     }
 }
 
+impl IntoArray for &MaskValues {
+    fn into_array(self) -> Array {
+        BoolArray::new(self.boolean_buffer().clone(), Nullability::NonNullable).into_array()
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use rstest::rstest;
+    use vortex_buffer::{buffer, Buffer};
+    use vortex_dtype::Nullability;
+    use vortex_mask::Mask;
 
-    use crate::array::BoolArray;
+    use crate::array::{BoolArray, PrimitiveArray};
     use crate::validity::Validity;
-    use crate::IntoArrayData;
+    use crate::IntoArray;
 
     #[rstest]
-    #[case(Validity::NonNullable, 5, &[2, 4], Validity::NonNullable, Validity::NonNullable)]
-    #[case(Validity::NonNullable, 5, &[2, 4], Validity::AllValid, Validity::NonNullable)]
-    #[case(Validity::NonNullable, 5, &[2, 4], Validity::AllInvalid, Validity::Array(BoolArray::from_iter([true, true, false, true, false]).into_array())
-    )]
-    #[case(Validity::NonNullable, 5, &[2, 4], Validity::Array(BoolArray::from_iter([true, false]).into_array()), Validity::Array(BoolArray::from_iter([true, true, true, true, false]).into_array())
-    )]
-    #[case(Validity::AllValid, 5, &[2, 4], Validity::NonNullable, Validity::AllValid)]
     #[case(Validity::AllValid, 5, &[2, 4], Validity::AllValid, Validity::AllValid)]
     #[case(Validity::AllValid, 5, &[2, 4], Validity::AllInvalid, Validity::Array(BoolArray::from_iter([true, true, false, true, false]).into_array())
     )]
     #[case(Validity::AllValid, 5, &[2, 4], Validity::Array(BoolArray::from_iter([true, false]).into_array()), Validity::Array(BoolArray::from_iter([true, true, true, true, false]).into_array())
     )]
-    #[case(Validity::AllInvalid, 5, &[2, 4], Validity::NonNullable, Validity::Array(BoolArray::from_iter([false, false, true, false, true]).into_array())
-    )]
     #[case(Validity::AllInvalid, 5, &[2, 4], Validity::AllValid, Validity::Array(BoolArray::from_iter([false, false, true, false, true]).into_array())
     )]
     #[case(Validity::AllInvalid, 5, &[2, 4], Validity::AllInvalid, Validity::AllInvalid)]
     #[case(Validity::AllInvalid, 5, &[2, 4], Validity::Array(BoolArray::from_iter([true, false]).into_array()), Validity::Array(BoolArray::from_iter([false, false, true, false, false]).into_array())
     )]
-    #[case(Validity::Array(BoolArray::from_iter([false, true, false, true, false]).into_array()), 5, &[2, 4], Validity::NonNullable, Validity::Array(BoolArray::from_iter([false, true, true, true, true]).into_array())
-    )]
     #[case(Validity::Array(BoolArray::from_iter([false, true, false, true, false]).into_array()), 5, &[2, 4], Validity::AllValid, Validity::Array(BoolArray::from_iter([false, true, true, true, true]).into_array())
     )]
     #[case(Validity::Array(BoolArray::from_iter([false, true, false, true, false]).into_array()), 5, &[2, 4], Validity::AllInvalid, Validity::Array(BoolArray::from_iter([false, true, false, true, false]).into_array())
@@ -553,18 +481,32 @@ mod tests {
     fn patch_validity(
         #[case] validity: Validity,
         #[case] len: usize,
-        #[case] positions: &[usize],
+        #[case] positions: &[u64],
         #[case] patches: Validity,
         #[case] expected: Validity,
     ) {
-        assert_eq!(validity.patch(len, positions, patches).unwrap(), expected);
+        let indices =
+            PrimitiveArray::new(Buffer::copy_from(positions), Validity::NonNullable).into_array();
+        assert_eq!(validity.patch(len, &indices, patches).unwrap(), expected);
     }
 
     #[test]
     #[should_panic]
     fn out_of_bounds_patch() {
         Validity::NonNullable
-            .patch(2, &[4], Validity::AllInvalid)
+            .patch(2, &buffer![4].into_array(), Validity::AllInvalid)
             .unwrap();
     }
+
+    #[test]
+    #[should_panic]
+    fn into_validity_nullable() {
+        Validity::from_mask(Mask::AllFalse(10), Nullability::NonNullable);
+    }
+
+    #[test]
+    #[should_panic]
+    fn into_validity_nullable_array() {
+        Validity::from_mask(Mask::from_iter(vec![true, false]), Nullability::NonNullable);
+    }
 }
