@@ -89,7 +89,7 @@ impl TemporalMetadata {
             TemporalMetadata::Timestamp(unit, Some(tz)) => Ok(TemporalJiff::Zoned(
                 Timestamp::UNIX_EPOCH
                     .checked_add(unit.to_jiff_span(v)?)?
-                    .intz(tz)?,
+                    .in_tz(tz)?,
             )),
         }
     }
@@ -201,6 +201,7 @@ mod tests {
 
     use crate::{TemporalMetadata, TimeUnit, TIMESTAMP_ID};
 
+    #[cfg_attr(miri, ignore)]
     #[test]
     fn test_roundtrip_metadata() {
         let meta: ExtMetadata =
