@@ -18,6 +18,19 @@ use crate::DType::*;
 #[derive(Debug, Clone, Copy, PartialEq, PartialOrd, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[cfg_attr(feature = "serde", serde(rename_all = "lowercase"))]
+#[cfg_attr(
+    feature = "rkyv",
+    derive(
+        rkyv::Archive,
+        rkyv::Portable,
+        rkyv::Serialize,
+        rkyv::Deserialize,
+        rkyv::bytecheck::CheckBytes,
+    ),
+    rkyv(as = PType),
+    bytecheck(crate = rkyv::bytecheck),
+)]
+#[repr(u8)]
 pub enum PType {
     /// An 8-bit unsigned integer
     U8,
@@ -60,6 +73,7 @@ pub trait NativePType:
     + FromPrimitive
     + ToBytes
     + TryFromBytes
+    + 'static
 {
     /// The PType that corresponds to this native type
     const PTYPE: PType;
@@ -147,6 +161,28 @@ macro_rules! match_each_native_ptype {
             PType::F32 => __with__! { f32 },
             PType::F64 => __with__! { f64 },
         }
+    });
+    ($self:expr,
+     integral: | $_:tt $integral_enc:ident | { $($integral_body:tt)* }
+     floating_point: | $_2:tt $floating_point_enc:ident | { $($floating_point_body:tt)* }
+    ) => ({
+        macro_rules! __with_integer__ {( $_ $integral_enc:ident ) => ( { $($integral_body)* } )}
+        macro_rules! __with_floating_point__ {( $_ $floating_point_enc:ident ) => ( { $($floating_point_body)* } )}
+        use $crate::PType;
+        use $crate::half::f16;
+        match $self {
+            PType::I8 => __with_integer__! { i8 },
+            PType::I16 => __with_integer__! { i16 },
+            PType::I32 => __with_integer__! { i32 },
+            PType::I64 => __with_integer__! { i64 },
+            PType::U8 => __with_integer__! { u8 },
+            PType::U16 => __with_integer__! { u16 },
+            PType::U32 => __with_integer__! { u32 },
+            PType::U64 => __with_integer__! { u64 },
+            PType::F16 => __with_floating_point__! { f16 },
+            PType::F32 => __with_floating_point__! { f32 },
+            PType::F64 => __with_floating_point__! { f64 },
+        }
     })
 }
 
@@ -165,9 +201,7 @@ macro_rules! match_each_integer_ptype {
             PType::U16 => __with__! { u16 },
             PType::U32 => __with__! { u32 },
             PType::U64 => __with__! { u64 },
-            PType::F16 =>  panic!("Unsupported ptype f16"),
-            PType::F32 =>  panic!("Unsupported ptype f32"),
-            PType::F64 =>  panic!("Unsupported ptype f64"),
+            other => panic!("Unsupported ptype {other}")
         }
     })
 }
@@ -183,7 +217,7 @@ macro_rules! match_each_unsigned_integer_ptype {
             PType::U16 => __with__! { u16 },
             PType::U32 => __with__! { u32 },
             PType::U64 => __with__! { u64 },
-            _ => panic!("Unsupported ptype {}", $self),
+            other => panic!("Unsupported ptype {other}"),
         }
     })
 }
@@ -199,7 +233,7 @@ macro_rules! match_each_float_ptype {
             PType::F16 => __with__! { f16 },
             PType::F32 => __with__! { f32 },
             PType::F64 => __with__! { f64 },
-            _ => panic!("Unsupported ptype {}", $self),
+            other => panic!("Unsupported ptype {other}"),
         }
     })
 }
