@@ -1,19 +1,80 @@
 use std::sync::Arc;
 
-use flatbuffers::{FlatBufferBuilder, WIPOffset};
+use flatbuffers::{FlatBufferBuilder, Follow, WIPOffset};
 use itertools::Itertools;
 use vortex_error::{vortex_bail, vortex_err, VortexError, VortexResult};
-use vortex_flatbuffers::{FlatBufferRoot, WriteFlatBuffer};
+use vortex_flatbuffers::{dtype as fbd, FlatBuffer, FlatBufferRoot, WriteFlatBuffer};
 
-use crate::{flatbuffers as fb, DType, ExtDType, ExtID, ExtMetadata, PType, StructDType};
+use crate::{
+    flatbuffers as fb, DType, ExtDType, ExtID, ExtMetadata, FieldDType, PType, StructDType,
+};
 
 mod project;
 pub use project::*;
 
-impl TryFrom<fb::DType<'_>> for DType {
+/// A lazily evaluated DType, parsed on access from an underlying flatbuffer.
+#[derive(Debug, Clone, PartialOrd, PartialEq, Eq, Hash)]
+pub(crate) struct ViewedDType {
+    /// Underlying flatbuffer
+    flatbuffer: FlatBuffer,
+    /// Location of the dtype data inside the underlying buffer
+    flatbuffer_loc: usize,
+}
+
+impl ViewedDType {
+    /// Create a [`ViewedDType`] from a buffer and a flatbuffer location
+    fn from_fb_loc(location: usize, buffer: FlatBuffer) -> Self {
+        Self {
+            flatbuffer: buffer,
+            flatbuffer_loc: location,
+        }
+    }
+
+    /// The viewed [`fbd::DType`] instance.
+    fn flatbuffer(&self) -> fbd::DType<'_> {
+        unsafe { fbd::DType::follow(self.flatbuffer.as_ref(), self.flatbuffer_loc) }
+    }
+
+    /// Returns the underlying shared buffer
+    fn buffer(&self) -> &FlatBuffer {
+        &self.flatbuffer
+    }
+}
+
+impl StructDType {
+    /// Creates a new instance from a flatbuffer-defined object and its underlying buffer.
+    pub fn from_fb(fb_struct: fbd::Struct_<'_>, buffer: FlatBuffer) -> VortexResult<Self> {
+        let names = fb_struct
+            .names()
+            .ok_or_else(|| vortex_err!("failed to parse struct names from flatbuffer"))?
+            .iter()
+            .map(|n| (*n).into())
+            .collect();
+
+        let dtypes = fb_struct
+            .dtypes()
+            .ok_or_else(|| vortex_err!("failed to parse struct dtypes from flatbuffer"))?
+            .iter()
+            .map(|dt| FieldDType::from(ViewedDType::from_fb_loc(dt._tab.loc(), buffer.clone())))
+            .collect::<Vec<_>>();
+
+        Ok(StructDType::from_fields(names, dtypes))
+    }
+}
+
+impl DType {
+    /// Create a new
+    pub fn try_from_view(fb: fbd::DType, buffer: FlatBuffer) -> VortexResult<Self> {
+        let vdt = ViewedDType::from_fb_loc(fb._tab.loc(), buffer);
+        Self::try_from(vdt)
+    }
+}
+
+impl TryFrom<ViewedDType> for DType {
     type Error = VortexError;
 
-    fn try_from(fb: fb::DType<'_>) -> Result<Self, Self::Error> {
+    fn try_from(vfdt: ViewedDType) -> Result<Self, Self::Error> {
+        let fb = vfdt.flatbuffer();
         match fb.type_type() {
             fb::Type::Null => Ok(Self::Null),
             fb::Type::Bool => Ok(Self::Bool(
@@ -47,9 +108,14 @@ impl TryFrom<fb::DType<'_>> for DType {
                 let fb_list = fb
                     .type__as_list()
                     .ok_or_else(|| vortex_err!("failed to parse list from flatbuffer"))?;
-                let element_dtype = Self::try_from(fb_list.element_type().ok_or_else(|| {
+
+                let list_element = fb_list.element_type().ok_or_else(|| {
                     vortex_err!("failed to parse list element type from flatbuffer")
-                })?)?;
+                })?;
+                let element_dtype = Self::try_from(ViewedDType::from_fb_loc(
+                    list_element._tab.loc(),
+                    vfdt.buffer().clone(),
+                ))?;
                 Ok(Self::List(
                     Arc::new(element_dtype),
                     fb_list.nullable().into(),
@@ -59,21 +125,10 @@ impl TryFrom<fb::DType<'_>> for DType {
                 let fb_struct = fb
                     .type__as_struct_()
                     .ok_or_else(|| vortex_err!("failed to parse struct from flatbuffer"))?;
-                let names = fb_struct
-                    .names()
-                    .ok_or_else(|| vortex_err!("failed to parse struct names from flatbuffer"))?
-                    .iter()
-                    .map(|n| (*n).into())
-                    .collect_vec()
-                    .into();
-                let dtypes: Vec<Self> = fb_struct
-                    .dtypes()
-                    .ok_or_else(|| vortex_err!("failed to parse struct dtypes from flatbuffer"))?
-                    .iter()
-                    .map(Self::try_from)
-                    .collect::<VortexResult<Vec<_>>>()?;
+                let struct_dtype = StructDType::from_fb(fb_struct, vfdt.buffer().clone())?;
+
                 Ok(Self::Struct(
-                    StructDType::new(names, dtypes),
+                    struct_dtype.into(),
                     fb_struct.nullable().into(),
                 ))
             }
@@ -86,15 +141,18 @@ impl TryFrom<fb::DType<'_>> for DType {
                         vortex_err!("failed to parse extension id from flatbuffer")
                     })?);
                 let metadata = fb_ext.metadata().map(|m| ExtMetadata::from(m.bytes()));
+                let storage_dtype = fb_ext.storage_dtype().ok_or_else(|| {
+                    vortex_err!(
+                InvalidSerde: "storage_dtype must be present on DType fbs message")
+                })?;
+                let storage_view =
+                    ViewedDType::from_fb_loc(storage_dtype._tab.loc(), vfdt.buffer().clone());
+
                 Ok(Self::Extension(Arc::new(ExtDType::new(
                     id,
-                    Arc::new(
-                        DType::try_from(fb_ext.storage_dtype().ok_or_else(|| {
-                            vortex_err!(
-                        InvalidSerde: "storage_dtype must be present on DType fbs message")
-                        })?)
-                        .map_err(|e| vortex_err!("failed to create DType from fbs message: {e}"))?,
-                    ),
+                    Arc::new(DType::try_from(storage_view).map_err(|e| {
+                        vortex_err!("failed to create DType from fbs message: {e}")
+                    })?),
                     metadata,
                 ))))
             }
@@ -104,6 +162,7 @@ impl TryFrom<fb::DType<'_>> for DType {
 }
 
 impl FlatBufferRoot for DType {}
+
 impl WriteFlatBuffer for DType {
     type Target<'a> = fb::DType<'a>;
 
@@ -152,7 +211,6 @@ impl WriteFlatBuffer for DType {
 
                 let dtypes = st
                     .dtypes()
-                    .iter()
                     .map(|dtype| dtype.write_flatbuffer(fbb))
                     .collect_vec();
                 let dtypes = Some(fbb.create_vector(&dtypes));
@@ -259,14 +317,18 @@ mod test {
     use std::sync::Arc;
 
     use flatbuffers::root;
-    use vortex_flatbuffers::FlatBufferToBytes;
+    use vortex_flatbuffers::{FlatBuffer, WriteFlatBufferExt};
 
     use crate::nullability::Nullability;
+    use crate::serde::flatbuffers::ViewedDType;
     use crate::{flatbuffers as fb, DType, PType, StructDType};
 
     fn roundtrip_dtype(dtype: DType) {
-        let bytes = dtype.with_flatbuffer_bytes(|bytes| bytes.to_vec());
-        let deserialized = DType::try_from(root::<fb::DType>(&bytes).unwrap()).unwrap();
+        let bytes = dtype.write_flatbuffer_bytes();
+        let root_fb = root::<fb::DType>(&bytes).unwrap();
+        let view = ViewedDType::from_fb_loc(root_fb._tab.loc(), FlatBuffer::from(bytes.clone()));
+
+        let deserialized = DType::try_from(view).unwrap();
         assert_eq!(dtype, deserialized);
     }
 
@@ -298,7 +360,8 @@ mod test {
                     DType::Utf8(Nullability::NonNullable),
                     DType::Primitive(PType::U16, Nullability::Nullable),
                 ],
-            ),
+            )
+            .into(),
             Nullability::NonNullable,
         ))
     }
