@@ -1,6 +1,6 @@
 use std::sync::Arc;
 
-use vortex_error::{vortex_err, VortexError, VortexResult};
+use vortex_error::{vortex_err, VortexError, VortexResult, VortexUnwrap};
 
 use crate::field::{Field, FieldPath};
 use crate::proto::dtype as pb;
@@ -30,7 +30,7 @@ impl TryFrom<&pb::DType> for DType {
                         .iter()
                         .map(TryInto::<Self>::try_into)
                         .collect::<VortexResult<Vec<_>>>()?,
-                ),
+                ).into(),
                 s.nullable.into(),
             )),
             DtypeType::List(l) => {
@@ -80,7 +80,7 @@ impl From<&DType> for pb::DType {
                 }),
                 DType::Struct(s, n) => DtypeType::Struct(pb::Struct {
                     names: s.names().iter().map(|s| s.as_ref().to_string()).collect(),
-                    dtypes: s.dtypes().iter().map(Into::into).collect(),
+                    dtypes: s.dtypes().map(|d| Self::from(&d)).collect(),
                     nullable: (*n).into(),
                 }),
                 DType::List(l, n) => DtypeType::List(Box::new(pb::List {
@@ -147,7 +147,9 @@ impl TryFrom<&pb::FieldPath> for FieldPath {
                 .ok_or_else(|| vortex_err!(InvalidSerde: "FieldPath part missing type"))?
             {
                 FieldType::Name(name) => path.push(Field::from(name.as_str())),
-                FieldType::Index(idx) => path.push(Field::from(*idx as usize)),
+                FieldType::Index(idx) => {
+                    path.push(Field::from(usize::try_from(*idx).vortex_unwrap()))
+                }
             }
         }
         Ok(FieldPath::from(path))
