@@ -9,7 +9,10 @@ pub mod python;
 
 use std::backtrace::Backtrace;
 use std::borrow::Cow;
+use std::convert::Infallible;
+use std::error::Error;
 use std::fmt::{Debug, Display, Formatter};
+use std::num::TryFromIntError;
 use std::ops::Deref;
 use std::{env, fmt, io};
 
@@ -52,10 +55,19 @@ impl Display for ErrString {
     }
 }
 
+impl From<Infallible> for VortexError {
+    fn from(_: Infallible) -> Self {
+        unreachable!()
+    }
+}
+
 /// The top-level error type for Vortex.
 #[derive(thiserror::Error)]
 #[non_exhaustive]
 pub enum VortexError {
+    /// A wrapped generic error
+    #[error("{0}\nBacktrace:\n{1}")]
+    Generic(Box<dyn Error + Send + Sync + 'static>, Backtrace),
     /// An index is out of bounds.
     #[error("index {0} out of bounds from {1} to {2}\nBacktrace:\n{3}")]
     OutOfBounds(usize, usize, usize, Backtrace),
@@ -193,6 +205,13 @@ pub enum VortexError {
         #[backtrace]
         url::ParseError,
     ),
+    /// Wrap errors for fallible integer casting.
+    #[error(transparent)]
+    TryFromInt(
+        #[from]
+        #[backtrace]
+        TryFromIntError,
+    ),
 }
 
 impl VortexError {
@@ -221,12 +240,16 @@ pub trait VortexUnwrap {
     fn vortex_unwrap(self) -> Self::Output;
 }
 
-impl<T> VortexUnwrap for VortexResult<T> {
+impl<T, E> VortexUnwrap for Result<T, E>
+where
+    E: Into<VortexError>,
+{
     type Output = T;
 
     #[inline(always)]
     fn vortex_unwrap(self) -> Self::Output {
-        self.unwrap_or_else(|err| vortex_panic!(err))
+        self.map_err(|err| err.into())
+            .unwrap_or_else(|err| vortex_panic!(err))
     }
 }
 
@@ -240,12 +263,16 @@ pub trait VortexExpect {
     fn vortex_expect(self, msg: &str) -> Self::Output;
 }
 
-impl<T> VortexExpect for VortexResult<T> {
+impl<T, E> VortexExpect for Result<T, E>
+where
+    E: Into<VortexError>,
+{
     type Output = T;
 
     #[inline(always)]
     fn vortex_expect(self, msg: &str) -> Self::Output {
-        self.unwrap_or_else(|e| vortex_panic!(e.with_context(msg.to_string())))
+        self.map_err(|err| err.into())
+            .unwrap_or_else(|e| vortex_panic!(e.with_context(msg.to_string())))
     }
 }
 
@@ -381,6 +408,23 @@ pub mod __private {
     }
 }
 
+#[cfg(feature = "rancor")]
+impl rancor::Source for VortexError {
+    fn new<T: Error + Send + Sync + 'static>(source: T) -> Self {
+        VortexError::Generic(Box::new(source), Backtrace::capture())
+    }
+}
+
+#[cfg(feature = "rancor")]
+impl rancor::Trace for VortexError {
+    fn trace<R>(self, trace: R) -> Self
+    where
+        R: Debug + Display + Send + Sync + 'static,
+    {
+        VortexError::Context(trace.to_string().into(), Box::new(self))
+    }
+}
+
 #[cfg(feature = "worker")]
 impl From<VortexError> for worker::Error {
     fn from(value: VortexError) -> Self {
