@@ -84,14 +84,36 @@ pub mod scalar;
 #[allow(unused_qualifications)]
 #[rustfmt::skip]
 #[path = "./generated/footer.rs"]
-/// A file format footer containining a serialized `vortex-serde` Layout.
+/// A file format footer containing a serialized `vortex-file` Layout.
 ///
 /// `footer.fbs`:
 /// ```flatbuffers
-#[doc = include_str!("../flatbuffers/vortex-serde/footer.fbs")]
+#[doc = include_str!("../flatbuffers/vortex-file/footer.fbs")]
 /// ```
 pub mod footer;
 
+#[cfg(feature = "layout")]
+#[allow(clippy::all)]
+#[allow(clippy::derive_partial_eq_without_eq)]
+#[allow(clippy::many_single_char_names)]
+#[allow(clippy::unwrap_used)]
+#[allow(dead_code)]
+#[allow(non_snake_case)]
+#[allow(non_camel_case_types)]
+#[allow(unsafe_op_in_unsafe_fn)]
+#[allow(unused_imports)]
+#[allow(unused_lifetimes)]
+#[allow(unused_qualifications)]
+#[rustfmt::skip]
+#[path = "./generated/layout.rs"]
+/// A serialized sequence of arrays, each with its buffers.
+///
+/// `layout.fbs`:
+/// ```flatbuffers
+#[doc = include_str!("../flatbuffers/vortex-layout/layout.fbs")]
+/// ```
+pub mod layout;
+
 #[cfg(feature = "ipc")]
 #[allow(clippy::all)]
 #[allow(clippy::derive_partial_eq_without_eq)]
@@ -115,6 +137,13 @@ pub mod footer;
 pub mod message;
 
 use flatbuffers::{root, FlatBufferBuilder, Follow, InvalidFlatbuffer, Verifiable, WIPOffset};
+use vortex_buffer::{ByteBuffer, ConstByteBuffer};
+
+/// We define a const-aligned byte buffer for flatbuffers with 8-byte alignment.
+///
+/// This is based on the assumption that the maximum primitive type is 8 bytes.
+/// See: <https://groups.google.com/g/flatbuffers/c/PSgQeWeTx_g>
+pub type FlatBuffer = ConstByteBuffer<8>;
 
 pub trait FlatBufferRoot {}
 
@@ -144,15 +173,18 @@ pub trait WriteFlatBuffer {
     ) -> WIPOffset<Self::Target<'fb>>;
 }
 
-pub trait FlatBufferToBytes {
-    fn with_flatbuffer_bytes<R, Fn: FnOnce(&[u8]) -> R>(&self, f: Fn) -> R;
+pub trait WriteFlatBufferExt: WriteFlatBuffer + FlatBufferRoot {
+    /// Write the flatbuffer into a [`FlatBuffer`].
+    fn write_flatbuffer_bytes(&self) -> FlatBuffer;
 }
 
-impl<F: WriteFlatBuffer + FlatBufferRoot> FlatBufferToBytes for F {
-    fn with_flatbuffer_bytes<R, Fn: FnOnce(&[u8]) -> R>(&self, f: Fn) -> R {
+impl<F: WriteFlatBuffer + FlatBufferRoot> WriteFlatBufferExt for F {
+    fn write_flatbuffer_bytes(&self) -> FlatBuffer {
         let mut fbb = FlatBufferBuilder::new();
         let root_offset = self.write_flatbuffer(&mut fbb);
         fbb.finish_minimal(root_offset);
-        f(fbb.finished_data())
+        let (vec, start) = fbb.collapse();
+        let end = vec.len();
+        FlatBuffer::align_from(ByteBuffer::from(vec).slice(start..end))
     }
 }
