@@ -9,26 +9,27 @@
 
 pub use buf::*;
 pub use dispatcher::*;
-#[cfg(feature = "tokio")]
+pub use io_buf::*;
 pub use limit::*;
 #[cfg(feature = "object_store")]
 pub use object_store::*;
 pub use read::*;
+pub use read_ranges::*;
 #[cfg(feature = "tokio")]
 pub use tokio::*;
 pub use write::*;
 
-mod aligned;
 mod buf;
 #[cfg(feature = "compio")]
 mod compio;
 mod dispatcher;
-#[cfg(feature = "tokio")]
+mod io_buf;
 mod limit;
 #[cfg(feature = "object_store")]
 mod object_store;
 pub mod offset;
 mod read;
+mod read_ranges;
 #[cfg(feature = "tokio")]
 mod tokio;
 mod write;
