@@ -5,12 +5,12 @@
 //! data buffers.
 //!
 //! This crate provides both in-memory message representations for holding IPC messages
-//! before/after serialization, as well as streaming readers and writers that sit on top
+//! before/after serialization, and streaming readers and writers that sit on top
 //! of any type implementing `VortexRead` or `VortexWrite` respectively.
 
+pub mod iterator;
 pub mod messages;
-pub mod stream_reader;
-pub mod stream_writer;
+pub mod stream;
 
 /// All messages in Vortex are aligned to start at a multiple of 64 bytes.
 ///
