@@ -1,8 +1,6 @@
 use vortex_array::aliases::hash_set::HashSet;
-use vortex_array::array::{Constant, ConstantArray, ConstantEncoding};
-use vortex_array::encoding::EncodingRef;
-use vortex_array::stats::ArrayStatistics;
-use vortex_array::{ArrayData, ArrayDef, IntoArrayData};
+use vortex_array::array::{ConstantArray, ConstantEncoding};
+use vortex_array::{Array, Encoding, EncodingId, IntoArray};
 use vortex_error::{VortexExpect, VortexResult};
 
 use crate::compressors::{CompressedArray, CompressionTree, EncodingCompressor};
@@ -13,14 +11,14 @@ pub struct ConstantCompressor;
 
 impl EncodingCompressor for ConstantCompressor {
     fn id(&self) -> &str {
-        Constant::ID.as_ref()
+        ConstantEncoding::ID.as_ref()
     }
 
     fn cost(&self) -> u8 {
         constants::CONSTANT_COST
     }
 
-    fn can_compress(&self, array: &ArrayData) -> Option<&dyn EncodingCompressor> {
+    fn can_compress(&self, array: &Array) -> Option<&dyn EncodingCompressor> {
         array
             .statistics()
             .compute_is_constant()?
@@ -29,7 +27,7 @@ impl EncodingCompressor for ConstantCompressor {
 
     fn compress<'a>(
         &'a self,
-        array: &ArrayData,
+        array: &Array,
         _like: Option<CompressionTree<'a>>,
         _ctx: SamplingCompressor<'a>,
     ) -> VortexResult<CompressedArray<'a>> {
@@ -42,11 +40,11 @@ impl EncodingCompressor for ConstantCompressor {
             )
             .into_array(),
             Some(CompressionTree::flat(self)),
-            Some(array.statistics()),
+            array,
         ))
     }
 
-    fn used_encodings(&self) -> HashSet<EncodingRef> {
-        HashSet::from([&ConstantEncoding as EncodingRef])
+    fn used_encodings(&self) -> HashSet<EncodingId> {
+        HashSet::from([ConstantEncoding::ID])
     }
 }
