@@ -6,6 +6,7 @@ pub const DEFAULT_MAX_COST: u8 = 3;
 pub const CHUNKED_COST: u8 = 0;
 pub const SPARSE_COST: u8 = 0;
 pub const STRUCT_COST: u8 = 0;
+pub const LIST_COST: u8 = 0;
 pub const VARBIN_COST: u8 = 0;
 
 // so fast that we can ignore the cost
@@ -21,9 +22,6 @@ pub const DATE_TIME_PARTS_COST: u8 = 1;
 pub const DICT_COST: u8 = 1;
 pub const FOR_COST: u8 = 1;
 pub const FSST_COST: u8 = 1;
-pub const ROARING_BOOL_COST: u8 = 1;
-pub const ROARING_INT_COST: u8 = 1;
-pub const RUN_END_BOOL_COST: u8 = 1;
 pub const RUN_END_COST: u8 = 1;
 
 // "expensive" encodings
