@@ -1,12 +1,13 @@
 use core::fmt::Display;
 use std::cmp::Ordering;
+use std::hash::{Hash, Hasher};
 use std::mem;
 
 use num_traits::NumCast;
 use paste::paste;
 use vortex_dtype::half::f16;
-use vortex_dtype::{NativePType, PType};
-use vortex_error::{vortex_err, VortexError};
+use vortex_dtype::{NativePType, PType, ToBytes};
+use vortex_error::{vortex_err, VortexError, VortexExpect};
 
 #[derive(Debug, Clone, Copy)]
 pub enum PValue {
@@ -26,14 +27,14 @@ pub enum PValue {
 impl PartialEq for PValue {
     fn eq(&self, other: &Self) -> bool {
         match (self, other) {
-            (Self::U8(s), Self::U8(o)) => s.is_eq(*o),
-            (Self::U16(s), Self::U16(o)) => s.is_eq(*o),
-            (Self::U32(s), Self::U32(o)) => s.is_eq(*o),
-            (Self::U64(s), Self::U64(o)) => s.is_eq(*o),
-            (Self::I8(s), Self::I8(o)) => s.is_eq(*o),
-            (Self::I16(s), Self::I16(o)) => s.is_eq(*o),
-            (Self::I32(s), Self::I32(o)) => s.is_eq(*o),
-            (Self::I64(s), Self::I64(o)) => s.is_eq(*o),
+            (Self::U8(s), o) => o.as_u64().vortex_expect("upcast") == *s as u64,
+            (Self::U16(s), o) => o.as_u64().vortex_expect("upcast") == *s as u64,
+            (Self::U32(s), o) => o.as_u64().vortex_expect("upcast") == *s as u64,
+            (Self::U64(s), o) => o.as_u64().vortex_expect("upcast") == *s,
+            (Self::I8(s), o) => o.as_i64().vortex_expect("upcast") == *s as i64,
+            (Self::I16(s), o) => o.as_i64().vortex_expect("upcast") == *s as i64,
+            (Self::I32(s), o) => o.as_i64().vortex_expect("upcast") == *s as i64,
+            (Self::I64(s), o) => o.as_i64().vortex_expect("upcast") == *s,
             (Self::F16(s), Self::F16(o)) => s.is_eq(*o),
             (Self::F32(s), Self::F32(o)) => s.is_eq(*o),
             (Self::F64(s), Self::F64(o)) => s.is_eq(*o),
@@ -42,17 +43,19 @@ impl PartialEq for PValue {
     }
 }
 
+impl Eq for PValue {}
+
 impl PartialOrd for PValue {
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
         match (self, other) {
-            (Self::U8(s), Self::U8(o)) => Some(s.total_compare(*o)),
-            (Self::U16(s), Self::U16(o)) => Some(s.total_compare(*o)),
-            (Self::U32(s), Self::U32(o)) => Some(s.total_compare(*o)),
-            (Self::U64(s), Self::U64(o)) => Some(s.total_compare(*o)),
-            (Self::I8(s), Self::I8(o)) => Some(s.total_compare(*o)),
-            (Self::I16(s), Self::I16(o)) => Some(s.total_compare(*o)),
-            (Self::I32(s), Self::I32(o)) => Some(s.total_compare(*o)),
-            (Self::I64(s), Self::I64(o)) => Some(s.total_compare(*o)),
+            (Self::U8(s), o) => Some((*s as u64).cmp(&o.as_u64().vortex_expect("upcast"))),
+            (Self::U16(s), o) => Some((*s as u64).cmp(&o.as_u64().vortex_expect("upcast"))),
+            (Self::U32(s), o) => Some((*s as u64).cmp(&o.as_u64().vortex_expect("upcast"))),
+            (Self::U64(s), o) => Some((*s).cmp(&o.as_u64().vortex_expect("upcast"))),
+            (Self::I8(s), o) => Some((*s as i64).cmp(&o.as_i64().vortex_expect("upcast"))),
+            (Self::I16(s), o) => Some((*s as i64).cmp(&o.as_i64().vortex_expect("upcast"))),
+            (Self::I32(s), o) => Some((*s as i64).cmp(&o.as_i64().vortex_expect("upcast"))),
+            (Self::I64(s), o) => Some((*s).cmp(&o.as_i64().vortex_expect("upcast"))),
             (Self::F16(s), Self::F16(o)) => Some(s.total_compare(*o)),
             (Self::F32(s), Self::F32(o)) => Some(s.total_compare(*o)),
             (Self::F64(s), Self::F64(o)) => Some(s.total_compare(*o)),
@@ -61,6 +64,30 @@ impl PartialOrd for PValue {
     }
 }
 
+impl Hash for PValue {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        self.to_le_bytes().hash(state);
+    }
+}
+
+impl ToBytes for PValue {
+    fn to_le_bytes(&self) -> &[u8] {
+        match self {
+            PValue::U8(v) => v.to_le_bytes(),
+            PValue::U16(v) => v.to_le_bytes(),
+            PValue::U32(v) => v.to_le_bytes(),
+            PValue::U64(v) => v.to_le_bytes(),
+            PValue::I8(v) => v.to_le_bytes(),
+            PValue::I16(v) => v.to_le_bytes(),
+            PValue::I32(v) => v.to_le_bytes(),
+            PValue::I64(v) => v.to_le_bytes(),
+            PValue::F16(v) => v.to_le_bytes(),
+            PValue::F32(v) => v.to_le_bytes(),
+            PValue::F64(v) => v.to_le_bytes(),
+        }
+    }
+}
+
 macro_rules! as_primitive {
     ($T:ty, $PT:tt) => {
         paste! {
@@ -250,8 +277,9 @@ impl TryFrom<PValue> for f16 {
     type Error = VortexError;
 
     fn try_from(value: PValue) -> Result<Self, Self::Error> {
-        // We serialize f16 as u16.
+        // We serialize f16 as u16, but this can also sometimes be narrowed down to u8 if e.g. == 0
         match value {
+            PValue::U8(u) => Some(Self::from_bits(u as u16)),
             PValue::U16(u) => Some(Self::from_bits(u)),
             PValue::F16(u) => Some(u),
             PValue::F32(f) => <Self as NumCast>::from(f),
@@ -310,6 +338,8 @@ impl Display for PValue {
 
 #[cfg(test)]
 mod test {
+    use std::cmp::Ordering;
+
     use vortex_dtype::half::f16;
     use vortex_dtype::PType;
 
@@ -332,4 +362,16 @@ mod test {
         assert!(!PValue::F16(f16::from_f32(10.0)).is_instance_of(&PType::U16));
         assert!(!PValue::F16(f16::from_f32(10.0)).is_instance_of(&PType::I16));
     }
+
+    #[test]
+    fn test_compare_different_types() {
+        assert_eq!(
+            PValue::I8(4).partial_cmp(&PValue::I8(5)),
+            Some(Ordering::Less)
+        );
+        assert_eq!(
+            PValue::I8(4).partial_cmp(&PValue::I64(5)),
+            Some(Ordering::Less)
+        );
+    }
 }
