@@ -1,4 +1,6 @@
-use vortex_buffer::{Buffer, BufferString};
+use std::sync::Arc;
+
+use vortex_buffer::{BufferString, ByteBuffer};
 use vortex_dtype::half::f16;
 use vortex_dtype::{DType, PType};
 use vortex_error::{vortex_bail, vortex_err, VortexError, VortexResult};
@@ -37,8 +39,7 @@ impl From<&ScalarValue> for pb::ScalarValue {
             ScalarValue(InnerScalarValue::List(v)) => {
                 let mut values = Vec::with_capacity(v.len());
                 for elem in v.iter() {
-                    // FIXME(DK): protobufs should probably also have inner vs non-inner?
-                    values.push(pb::ScalarValue::from(&ScalarValue(elem.clone())));
+                    values.push(pb::ScalarValue::from(elem));
                 }
                 pb::ScalarValue {
                     kind: Some(Kind::ListValue(ListValue { values })),
@@ -143,18 +144,18 @@ fn deserialize_scalar_value(dtype: &DType, value: &pb::ScalarValue) -> VortexRes
         Kind::Uint64Value(v) => Ok(ScalarValue(InnerScalarValue::Primitive(PValue::U64(*v)))),
         Kind::FloatValue(v) => Ok(ScalarValue(InnerScalarValue::Primitive(PValue::F32(*v)))),
         Kind::DoubleValue(v) => Ok(ScalarValue(InnerScalarValue::Primitive(PValue::F64(*v)))),
-        Kind::StringValue(v) => Ok(ScalarValue(InnerScalarValue::BufferString(
+        Kind::StringValue(v) => Ok(ScalarValue(InnerScalarValue::BufferString(Arc::new(
             BufferString::from(v.clone()),
-        ))),
-        Kind::BytesValue(v) => Ok(ScalarValue(InnerScalarValue::Buffer(Buffer::from(
-            v.as_slice(),
+        )))),
+        Kind::BytesValue(v) => Ok(ScalarValue(InnerScalarValue::Buffer(Arc::new(
+            ByteBuffer::from(v.clone()),
         )))),
         Kind::ListValue(v) => {
             let mut values = Vec::with_capacity(v.values.len());
             match dtype {
                 DType::Struct(structdt, _) => {
-                    for (elem, dtype) in v.values.iter().zip(structdt.dtypes().iter()) {
-                        values.push(deserialize_scalar_value(dtype, elem)?);
+                    for (elem, dtype) in v.values.iter().zip(structdt.dtypes()) {
+                        values.push(deserialize_scalar_value(&dtype, elem)?);
                     }
                 }
                 DType::List(elementdt, _) => {
@@ -164,9 +165,7 @@ fn deserialize_scalar_value(dtype: &DType, value: &pb::ScalarValue) -> VortexRes
                 }
                 _ => vortex_bail!("invalid dtype for list value {}", dtype),
             }
-            Ok(ScalarValue(InnerScalarValue::List(
-                values.iter().map(|x| x.0.clone()).collect(),
-            )))
+            Ok(ScalarValue(InnerScalarValue::List(values.into())))
         }
     }
 }
@@ -215,7 +214,7 @@ mod test {
     fn test_buffer() {
         round_trip(Scalar::new(
             DType::Binary(Nullability::Nullable),
-            ScalarValue(InnerScalarValue::Buffer(vec![1, 2, 3].into())),
+            ScalarValue(InnerScalarValue::Buffer(Arc::new(vec![1, 2, 3].into()))),
         ));
     }
 
@@ -223,8 +222,8 @@ mod test {
     fn test_buffer_string() {
         round_trip(Scalar::new(
             DType::Utf8(Nullability::Nullable),
-            ScalarValue(InnerScalarValue::BufferString(BufferString::from(
-                "hello".to_string(),
+            ScalarValue(InnerScalarValue::BufferString(Arc::new(
+                BufferString::from("hello".to_string()),
             ))),
         ));
     }
@@ -238,8 +237,8 @@ mod test {
             ),
             ScalarValue(InnerScalarValue::List(
                 vec![
-                    InnerScalarValue::Primitive(42i32.into()),
-                    InnerScalarValue::Primitive(43i32.into()),
+                    ScalarValue(InnerScalarValue::Primitive(42i32.into())),
+                    ScalarValue(InnerScalarValue::Primitive(43i32.into())),
                 ]
                 .into(),
             )),
