@@ -1,11 +1,13 @@
 use std::fmt::Formatter;
+use std::sync::Arc;
 
 use serde::de::{Error, SeqAccess, Visitor};
+use serde::ser::SerializeSeq;
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
-use vortex_buffer::BufferString;
+use vortex_buffer::{BufferString, ByteBuffer};
 
 use crate::pvalue::PValue;
-use crate::value::{InnerScalarValue, ScalarValue};
+use crate::{InnerScalarValue, ScalarValue};
 
 impl Serialize for ScalarValue {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
@@ -25,9 +27,19 @@ impl Serialize for InnerScalarValue {
             Self::Null => ().serialize(serializer),
             Self::Bool(b) => b.serialize(serializer),
             Self::Primitive(p) => p.serialize(serializer),
-            Self::Buffer(buffer) => buffer.as_ref().serialize(serializer),
-            Self::BufferString(buffer) => buffer.as_str().serialize(serializer),
-            Self::List(l) => l.serialize(serializer),
+            // NOTE: we explicitly handle the serialization of bytes, strings and lists so as not
+            //  to create ambiguities amongst them. The serde data model has specific representations
+            //  of binary data, UTF-8 strings and sequences.
+            //  See https://serde.rs/data-model.html.
+            Self::Buffer(buffer) => serializer.serialize_bytes(buffer.as_slice()),
+            Self::BufferString(buffer) => serializer.serialize_str(buffer.as_str()),
+            Self::List(l) => {
+                let mut seq = serializer.serialize_seq(Some(l.len()))?;
+                for item in l.iter() {
+                    seq.serialize_element(item)?;
+                }
+                seq.end()
+            }
         }
     }
 }
@@ -126,16 +138,18 @@ impl<'de> Deserialize<'de> for ScalarValue {
             where
                 E: Error,
             {
-                Ok(ScalarValue(InnerScalarValue::BufferString(
+                Ok(ScalarValue(InnerScalarValue::BufferString(Arc::new(
                     BufferString::from(v.to_string()),
-                )))
+                ))))
             }
 
             fn visit_bytes<E>(self, v: &[u8]) -> Result<Self::Value, E>
             where
                 E: Error,
             {
-                Ok(ScalarValue(InnerScalarValue::Buffer(v.to_vec().into())))
+                Ok(ScalarValue(InnerScalarValue::Buffer(Arc::new(
+                    ByteBuffer::from(v.to_vec()),
+                ))))
             }
 
             fn visit_unit<E>(self) -> Result<Self::Value, E>
@@ -153,9 +167,7 @@ impl<'de> Deserialize<'de> for ScalarValue {
                 while let Some(e) = seq.next_element::<ScalarValue>()? {
                     elems.push(e);
                 }
-                Ok(ScalarValue(InnerScalarValue::List(
-                    elems.iter().map(|x| x.0.clone()).collect(),
-                )))
+                Ok(ScalarValue(InnerScalarValue::List(elems.into())))
             }
         }
 
@@ -184,3 +196,46 @@ impl Serialize for PValue {
         }
     }
 }
+
+impl<'de> Deserialize<'de> for PValue {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        ScalarValue::deserialize(deserializer)
+            .and_then(|scalar| scalar.0.as_pvalue().map_err(Error::custom))
+            .and_then(|pvalue| {
+                pvalue.ok_or_else(|| Error::custom("Expected a non-null primitive scalar value"))
+            })
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use std::mem::discriminant;
+    use std::sync::Arc;
+
+    use flexbuffers::{FlexbufferSerializer, Reader};
+    use rstest::rstest;
+    use vortex_dtype::{Nullability, PType};
+
+    use super::*;
+    use crate::Scalar;
+
+    #[rstest]
+    #[case(Scalar::binary(ByteBuffer::copy_from(b"hello"), Nullability::NonNullable).into_value())]
+    #[case(Scalar::utf8("hello", Nullability::NonNullable).into_value())]
+    #[case(Scalar::primitive(1u8, Nullability::NonNullable).into_value())]
+    #[case(Scalar::list(Arc::new(PType::U8.into()), vec![Scalar::primitive(1u8, Nullability::NonNullable)], Nullability::NonNullable).into_value())]
+    fn test_scalar_value_serde_roundtrip(#[case] scalar_value: ScalarValue) {
+        let mut serializer = FlexbufferSerializer::new();
+        scalar_value.serialize(&mut serializer).unwrap();
+        let written = serializer.take_buffer();
+        let reader = Reader::get_root(written.as_ref()).unwrap();
+        let scalar_read_back = ScalarValue::deserialize(reader).unwrap();
+        assert_eq!(
+            discriminant(&scalar_value.0),
+            discriminant(&scalar_read_back.0)
+        );
+    }
+}
